/*
** blit32.c
**
** Simple blitter to update a destination image part with a source
*/


#include "blit32.h"
#include "baseinc.h"


/*Blit32
**Blits from source to destination, respects source's alpha,
**dest's transparency is zeroed if any
**Note: does not check anything
*/
void Blit32(tImage* src, tImage* dst, adint xs, adint ys, adint xd, adint yd, adint w, adint h)
{
 dword* isrc=(src->i)+xs+(src->p)*ys,
      * idst=(dst->i)+xd+(dst->p)*yd;
 adint  ays=(src->p)-w,
        ayd=(dst->p)-w;
 dword oc=0,cp=0,cc,t;
 adint i;

 while (h>0){
 
  for(i=0;i<w;i++){
   cc=*isrc;
   t=cc>>24;
   if(t){
    if(t!=255){
     if(cp!=cc){
      oc=(((cc & 0xFF00FF)*(255-t))&0xFF00FF00)+(((cc & 0xFF00)*(255-t))&0xFF0000);
      cp=cc;
     }
     *idst=(( ((((*idst) &0xFF00FF)*t)&0xFF00FF00)+
              ((((*idst) &0xFF00)*t)&0xFF0000)+
           oc)>>8);     
    }
   }else /*Non - transparent*/
    *idst=cc;
   isrc++;
   idst++;
  }
  isrc+=ays;
  idst+=ayd;
  h--;
 }
}


/*Mix32
**Mixes src on dst: both may be images with transparency data: the result then
**will also have appropriate transparency data. The two image's x,y,and pitch
**has to be identical (Does not check!). m is for how much from src should be
**taken: 0: nothing, 256: all*/
void Mix32(tImage* src, tImage* dst, dword m)
{
 dword* isrc=(src->i),
      * idst=(dst->i);
 adint  ay=(src->p)-(src->x);
 adint i,j;

 if (m==0) return;
 if (m==256){
  for (j=0;j<(src->y);j++){
   for (i=0;i<(src->x);i++) idst[i]=isrc[i];
   isrc+=src->p;
   idst+=dst->p;
  }
  return;
 }
 
 /*Hard work is necessary: mixing*/
 for (j=0;j<(src->y);j++){
  for (i=0;i<(src->x);i++){
   *idst=(( ((( (*isrc)     & 0xFF00FF)*m) & 0xFF00FF00) + ((( (*idst)     & 0xFF00FF)*(256-m)) & 0xFF00FF00) )>>8)+
            (((((*isrc)>>8) & 0xFF00FF)*m) & 0xFF00FF00) + (((((*idst)>>8) & 0xFF00FF)*(256-m)) & 0xFF00FF00);
   isrc++; idst++;
  }
  isrc+=ay;
  idst+=ay;
 }
 
}


/*Clears a region to the given color*/
void Clear32(adint x, adint y, adint w, adint h, dword col, tImage* dest)
{
 adint t1,t2;
 dword* ti;
 ti=(dest->i)+x+y*(dest->p);
 for(t1=0;t1<h;t1++){
  for (t2=0;t2<w;t2++) ti[t2]=0;
  ti+=dest->p;
 }
}
