/*
** ASCIICmp
**
** Tiny program to compress Kye or RuChKye level data for inclusion in
** the RuChKye binary (Or truly any other data)
**
** Format:
**  0-127: ASCII letter
**  128-255: Repeat following:
**   7th bit is 1
**   6-5 bit: sequence length: 1-4 (characters)
**   4-0 bit: repeat count: 1-32 (repeats)
**
** Calling:
**  asciicmp file1 file2 ...
**  compresses the input files & outputs hex dump
*/

#include <stdio.h>
#include <stdlib.h>




/*Returns 6(+1) byte hex string:
** " 0xHH,"*/
char hexout_buf[7]={' ','0','x','4','3',',',0};
char* HexOut(unsigned char c)
{
 if ((c>>4)<10) hexout_buf[3]='0'+(c>>4);
 else hexout_buf[3]='A'+(c>>4)-10;
 if ((c&15)<10) hexout_buf[4]='0'+(c&15);
 else hexout_buf[4]='A'+(c&15)-10;
 return hexout_buf;
}


/*Prints out a character with the most appropriate representation,
**also starts new line when encountering a new line character*/
void CharOut(unsigned char c)
{
 if ((c>=32) && (c!=127) && (c!=255)){
  if ((c!='\'') && (c!='\\')) printf(" '%c', ",c);
  else printf(" '\\%c',",c);
 }else if (c=='\n') printf(" '\\n',\n");
 else if (c=='\r') printf(" '\\r',");
 else if (c=='\t') printf(" '\\t',");
 else printf("%s",HexOut(c));
}



int main (int argn, char* argv[])
{
 FILE *fh;
 int carg=1;
 int flen;
 int osize=0,oasize=0;
 int t1,t2,t3,t4,t5,t6;
 unsigned char *in, *inorig;

 /*No files given: output help*/
 if ((argn==1) || ((argn==2) && (argv[1][0]=='-'))){
  printf("Tiny ASCII compressor 1.00\nRunning Cheetah, August, 2006\nConverts mainly Kye levels to some includeable hex dumps\n\nUsage:\n\n asciicmp file1 [file2 [...]]\n\n");
  return 0;
 }
 
 /*Cycling through output files & generating stuff*/
 while (carg<argn){

  if (NULL==(fh=fopen(argv[carg],"rb"))) printf("/*Unable to open: %s*/\n\n",argv[carg]);
  else{
   
   /*Reading in the file*/
   t2=0;
   if (fseek(fh,0,SEEK_END)!=0) t2=1; /*To get file size*/
   else{
    flen=(int)ftell(fh);
    fseek(fh,0,SEEK_SET); /*Seek back to beginning*/
    if (NULL==(in=malloc(flen))) t2=1;
    else if (fread(in,(size_t)(flen),1,fh)==0){ t2=1; free(in); }
   }
   fclose(fh);
   inorig=in;

   if (t2){ printf("/*Could not read in: %s*/\n\n",argv[carg]); carg++; continue; }

   printf("/*%s data begins*/\n\n",argv[carg]);
   osize=0; /*This file's output size*/
   
   /*Start compressing...*/
   while(1){
   
    /*Increasing the sequence length try to look forth from the current
    **position if repeating pattern can be found*/
    t1=1; /*Sequence length*/
    /*t2=0; Pattern repeat count*/
    t3=1; /*Sequence length determined to be the best*/
    t4=1; /*It's total length in bytes (what will be squeezed in shorter)*/
    while (t1<=4){
     t2=0;
     do{
      t2++; /*One longer pattern than this suitable?*/
      t5=0; /*Repeated?*/
      if (((t2+1)*t1)<=flen){ /*Is the input long enough?*/
       t5=1;
       for (t6=0;t6<t1;t6++) if (in[t6]!=in[t6+(t1*t2)]) t5=0; /*Not repeating!*/
      }
     }while(t5 && (t2<32));
     /*Now compare with the pervious compression decided to be the best*/
     /*(Taking in account that the sequence once has to be stored)*/
     if ((t2-1)*t1>(t4-t3)){ /*Better!*/
      t3=t1;
      t4=t2*t1;
     }
     t1++; /*Try out one longer sequence*/
    }
    t2=t4/t3; /*Gain back sequence length*/
    
    /*Now the best pattern & length combination was got, write it out
    **appropriately and step further!*/
    if ((t3==1) && (t4==1) && ((*in)<128)){ /*One character: just output it*/
     CharOut(*in);
     in++;
     flen--;
     osize++;
    }else{ /*Create the appropriate sequence header & output*/
     printf("%s",HexOut(0x80|((t3-1)<<5)|(t2-1)));
     for(t1=0;t1<t3;t1++) CharOut(in[t1]);
     in+=t4;
     flen-=t4;
     osize+=t3+1;
    }
    
    /*Gone further, ready for the next chunk of data...*/
    if (flen==0) break; /*Of course only if there is anything remaining*/
   }
   
   /*Data written, now write it's size!*/
   printf("\n\n/*End of %d bytes*/\n\n\n",osize);
   oasize+=osize;

   /*And of course take care of the buffer*/
   free(inorig);
  }
  
  carg++;
 }
 
 if (oasize) printf("\n/*A total of %d bytes were written*/\n\n\n",oasize);
 
 return 0;
 
}
