/*
** gamemain.c
**
** The game loop: realizes the actual game
*/



#include "baseinc.h"
#include "screen.h"
#include <SDL/SDL.h>
#include "kyemapmn.h"
#include "kyeinter.h"
#include "kyeobjs.h"
#include "timer.h"
#include "ppcres.h"
#include "gamemain.h"
#include "geffects.h"
#include "cvstr.h"
#include "text.h"
#include "cursor.h"
#include "blit32.h"
#include "intens.h"


/*The language elements used*/
byte* game_lang[8+8]={
 "Restart",
 "Next lv.",
 "Prev. lv.",
 "Skip mv.",
 "Exit",
 "Running Cheetah Kye - programmed by Jubatian (Spotted Cheetah); August, 2006; original idea & game (in 1992) by\
 Colin Garbutt.\
 The basic game idea is to collect all diamonds avoiding the traps or problems set by the objects moving around\
 (or standing still). If you liked this game, like with the\
 original, please donate to charity. Remember, there are many ones in this world who may never be able to even see a\
 computer, and many animals who had and have to die for human's good, they all need just a little help.\
 Not much. Not millions. Just a little. If everyone had given, it would add up, and help the good. So play, and\
 consider these if you liked this piece of game... ... ... WRAP ...",
 /*Scrolltext*/
 NULL,   /*Level name*/
 NULL,   /*Level hint*/
 NULL,NULL,NULL,NULL, /*For the 8 help boxes*/
 NULL,NULL,NULL,NULL
};
/*Colors used to draw elements*/
dword game_colors[3]={
 0x000000,   /*Background color for text boxes*/
 0xEECC44,   /*Mouse cursor color*/
 0x66CC22    /*Text color*/
};



/*Game_SetLang
**Sets language - specific texts
**Gets a pointer to six pointers which point to these texts in order
**(Restart, Next lv, Prev lv, Skip mv, Exit, the scrolltext)
**NULL does not change*/
void Game_SetLang(byte** lang)
{
 adint i;
 if (lang==NULL) return;
 for (i=0; i<6; i++) if (lang[i]!=NULL) game_lang[i]=lang[i];
}
/*Game_SetCol
**Gets three colors: background, mouse, and text color*/
void Game_SetCol(dword bg, dword m, dword t)
{
 game_colors[0]=bg;
 game_colors[1]=m;
 game_colors[2]=t;
}



/*Returns true if only one bit is set in the given value*/
adint Game_OneBit(adint in)
{
 adint c=1;
 while (c<0x10000){
  if (in==c) return 1;
  c<<=1;
 }
 return 0;
}



/*Outputs remaining Kyes (>4: numeric; <=4: Kyes)
**passing NULL as dest asks for lock & set & update*/
void Game_Kyes(adint kyen, tImage* dest)
{
 adint i,l=0;
 byte tstr[5]={'x',0,0,0,0};
 if (dest==NULL){ dest=Screen_Lock(); if (dest==NULL) return; l=1; }
 Clear32(545,120,80,16,game_colors[0],dest);
 if (kyen<=4){
  for (i=0;i<kyen;i++) Kye_O_Kye(0,0,550+i*18,120,dest);
 }else{
  Kye_O_Kye(0,0,550,120,dest);
  CvStr(kyen,3,tstr+1);
  Text_Print(tstr,568,120,game_colors[2],dest);
 }
 if (l){ Screen_Unlock(); Screen_Update(545,120,80,16); }
}

/*Outputs remaining diamonds (>4: numeric; <=4: diamonds)*/
void Game_Diamonds(adint din, tImage* dest)
{
 adint i,l=0;
 byte tstr[5]={'x',0,0,0,0};
 if (dest==NULL){ dest=Screen_Lock(); if (dest==NULL) return; l=1; }
 Clear32(545,140,80,16,game_colors[0],dest);
 if (din<=4){
  for (i=0;i<din;i++) Kye_O_Diamond(0,0,550+i*18,140,dest);
 }else{
  Kye_O_Diamond(0,0,550,140,dest);
  CvStr(din,3,tstr+1);
  Text_Print(tstr,568,140,game_colors[2],dest);
 }
 if (l){ Screen_Unlock(); Screen_Update(545,140,80,16); }
}


/*Centers a short text on the given area (writes with the text color)*/
void Game_CT(byte* tstr,adint x, adint y, adint w, tImage* dest)
{
 adint l=Text_Print(tstr,0,0,0,NULL);
 Text_Print(tstr,x+((w-l)>>1),y,game_colors[2],dest);
}


/*Prints the scrolltext at it's appropriate location*/
sadint game_scrolltext_x=600;
void Game_ScrollText(adint add)
{
 tImage *dest=Screen_Lock(),ti;
 sadint t1,t2;
 
 if (dest==NULL) return;
 if (game_lang[5]==NULL){ Screen_Unlock(); return; }

 MakeWindow(dest,20,444,600,16,&ti);
 Clear32(16,0,600-32,16,game_colors[0],&ti);
 t1=Text_Print(game_lang[5],game_scrolltext_x,0,game_colors[2],&ti);
 t2=Text_Print("<",0,0,0,NULL);
 Clear32(0,0,16,16,game_colors[0],&ti);
 Clear32(600-16,0,16,16,game_colors[0],&ti);
 Text_Print(">",0,0,game_colors[2],&ti);
 Text_Print("<",600-t2,0,game_colors[2],&ti);
 
 Screen_Unlock();
 Screen_Update(20,444,600,16);

 if (t1<(-game_scrolltext_x)) game_scrolltext_x=600;
 game_scrolltext_x-=add;
}


/*After setting up a map this will run the game.*/
/*-1 tells if some critical fault happened,
**positive values (and zero) are for normal termination
** GAME_HALT: Exit from the whole program requested
** GAME_EXIT: Exit from the game requested (ESC)
** GAME_DIED: Kye died, take care of substracting from lives ect. (ENTER)
** GAME_DIAM: All diamonds were collected
** GAME_NEXT: Request for next level (PgUp)
** GAME_PREV: Request for prev level (PgDn)
**Keys:
** Arrows; numpad: move
** Numpad 5: Skip a move (in turn - based mode)
**levtxt is an array of 10 pointers: level name, level hint, and 8 help block
**texts. NULL can be given to leave empty an area
**in flags the number of lives can be passed (to output it on screen), and
**the GAME_NOUPD flag to ask to not redraw the decorating borders
*/

#define GAME_MXMIN (545+CURSOR_HX)
#define GAME_MXMAX (545+80-CURSOR_WIDTH+CURSOR_HX)
#define GAME_MYMIN (115+CURSOR_HY)
#define GAME_MYMAX (115+284-CURSOR_WIDTH+CURSOR_HY)
adint game_main_frame=0;
sadint Game_Main(byte** levtxt,adint flags)
{

 SDL_Event event; /*The event got from the queue*/
 adint keyc=0,keycc=0,keyp=0,keyclr=1,lkey=0,fr=0;
 adint diamonds=0, t1, t2, ret=-1;
 adint poll=1;
 adint oldtimer=Timer_Get();
 sadint mx=585, my=257;
 adint mouc=0;
 tKyeObject obj;
 tImage* scr, ti;

 if (levtxt!=NULL)
  for (t1=0; t1<10; t1++) if (levtxt[t1]!=NULL) game_lang[t1+6]=levtxt[t1];
 
 /*Count diamonds on the map*/
 while ((t1=Kye_NextObj(&obj))!=-1) if (obj.t==KYE_O_DIAMOND) diamonds++;
 
 /*Draw gamefield borders & stuff*/
 if (flags & GAME_NOUPD){
  Game_Kyes(flags & 0xFF,NULL);
  Game_Diamonds(diamonds,NULL);
 }else if ((scr=Screen_Lock())!=NULL){
  Effect_3DBorderT(0xBBFFFFFF,0xBB000000,5,0,0,640,480,scr);
  Effect_3DBorderT(0xBB000000,0xBBFFFFFF,5,10,10,522,394,scr);
  Effect_3DBorderT(0xBB000000,0xBBFFFFFF,5,540,10,90,90,scr);
  Clear32(545,15,80,80,0,scr);
  PPC_Iconv_Disp(ppc_ruchkye,545,15,256,0,scr);
  Effect_3DBorderT(0xBB000000,0xBBFFFFFF,5,540,110,90,294,scr);
  Clear32(545,115,80,284,game_colors[0],scr);
  Game_Kyes(flags & 0xFF,scr);
  Game_Diamonds(diamonds,scr);
  /*Controls - Kye*/
  Game_CT("U",577,160,16,scr);
  Game_CT("7",562,175,16,scr);
  Game_CT("8",577,175,16,scr);
  Game_CT("9",592,175,16,scr);
  Game_CT("L",547,190,16,scr);
  Game_CT("4",562,190,16,scr);
  Kye_O_Kye(0,0,577,190,scr);
  Game_CT("6",592,190,16,scr);
  Game_CT("R",607,190,16,scr);
  Game_CT("1",562,205,16,scr);
  Game_CT("2",577,205,16,scr);
  Game_CT("3",592,205,16,scr);
  Game_CT("D",577,220,16,scr);
  /*Controls - misc*/
  Text_Print("Enter:",550,240,game_colors[2],scr);
  Text_Print(game_lang[0],560,255,game_colors[2],scr);
  Text_Print("PgUp:",550,270,game_colors[2],scr);
  Text_Print(game_lang[1],560,285,game_colors[2],scr);
  Text_Print("PgDn:",550,300,game_colors[2],scr);
  Text_Print(game_lang[2],560,315,game_colors[2],scr);
  Text_Print("5, Spc:",550,330,game_colors[2],scr);
  Text_Print(game_lang[3],560,345,game_colors[2],scr);
  Text_Print("ESC:",550,360,game_colors[2],scr);
  Text_Print(game_lang[4],560,375,game_colors[2],scr);
  /*Bottom areas - Level description & such*/
  Effect_3DBorderT(0xBB000000,0xBBFFFFFF,5,10,412,620,58,scr);
  Clear32(15,417,610,48,game_colors[0],scr);
  MakeWindow(scr,20,422,600,38,&ti);
  if (game_lang[6]!=NULL) t1=Text_Print(game_lang[6],0,0,game_colors[2],&ti)+20;
  else t1=0;
  if (game_lang[7]!=NULL) Text_Print(game_lang[7],t1,0,game_colors[2],&ti);
  Game_ScrollText(0);
  /*Playfield*/
  Kye_Update();
  /*Done*/
  Screen_Unlock();
  Screen_Update(0,0,640,480);
 }
 /*Get initial mouse background*/
 if ((scr=Screen_Lock())!=NULL){
  Blit32(scr,&cursorbk,mx-CURSOR_HX,my-CURSOR_HY,0,0,CURSOR_WIDTH,CURSOR_HEIGHT);
  Screen_Unlock();
 }
  
 
 if (Timer_Set(20)) return GAME_HALT;
 Kye_Interact_Reset(); /*For proper interaction & to always reproduce the
                       **same movements (maybe further saved replays?)*/

 /*The main loop: It is more straightforward to exit from it with a suitable
 **break at the proper place*/
 while(1){


  if (poll) if (SDL_PollEvent(&event)==0) poll=0; /*Got events*/
  
  if (poll==0){ /*Spare time: frame may be processed*/
   if (game_main_frame>0){
    Game_ScrollText(game_main_frame);
    Kye_Update();
    game_main_frame=0;
   }
   /*Wait for next events...*/
   if (SDL_WaitEvent(&event)==0) ret=GAME_HALT; /*Exit on event read fault*/
   poll=1;
  }
  
  if ((event.type==SDL_USEREVENT) && (event.user.code==TIMER_CODE)){
   /*Timer event: process a frame*/
   game_main_frame++;

   if (Kye_InteractFrame()){
    if ((lkey>6) && (Game_OneBit(keycc|keyc))){ /*Maybe wished a diagonal move... Wait next frame*/
     keyp=0; keyclr=0;
    }else{
     keyp=keycc|keyc; keyclr=1;
    }
   }
   
   if ((t1=Kye_Interact(keyp | mouc)) & 0x1){
    if (keyclr) keycc=0; /*The keyc & keycc mechanism results in more user - friendly handling
                         **by catching even too short presses as well*/
    mouc=0;
    if (t1 & 0x10){diamonds--;
                   Game_Diamonds(diamonds,NULL);
                   if (diamonds==0) ret=GAME_DIAM;}
    if (t1 & 0x8)  ret=GAME_DIED;
    fr=0;
    lkey=0;
   }
   fr++;

  }else if (event.type==SDL_KEYDOWN){
   /*Some key was pressed. Which?*/
   switch (event.key.keysym.sym){
    case SDLK_LEFT:    keyc|=8;   keycc|=8;   lkey=fr; break;
    case SDLK_RIGHT:   keyc|=2;   keycc|=2;   lkey=fr; break;
    case SDLK_UP:      keyc|=1;   keycc|=1;   lkey=fr; break;
    case SDLK_DOWN:    keyc|=4;   keycc|=4;   lkey=fr; break;
    case SDLK_KP7:     keyc|=8|1; keycc|=8|1; lkey=0;  break; /*No wait for keypad: has diagonal moves!*/
    case SDLK_KP8:     keyc|=1;   keycc|=1;   lkey=0;  break;
    case SDLK_KP9:     keyc|=1|2; keycc|=1|2; lkey=0;  break;
    case SDLK_KP4:     keyc|=8;   keycc|=8;   lkey=0;  break;
    case SDLK_KP5: case SDLK_SPACE: keyc|=0xF; keycc|=0xF; lkey=0;  break; /*So will jump frame in turn - based mode*/
    case SDLK_KP6:     keyc|=2;   keycc|=2;   lkey=0;  break;
    case SDLK_KP1:     keyc|=8|4; keycc|=8|4; lkey=0;  break;
    case SDLK_KP2:     keyc|=4;   keycc|=4;   lkey=0;  break;
    case SDLK_KP3:     keyc|=4|2; keycc|=4|2; lkey=0;  break;
    case SDLK_ESCAPE:  ret=GAME_EXIT; break; /*Exit game*/
    case SDLK_RETURN:  ret=GAME_DIED; break; /*Self - destruct*/
    case SDLK_PAGEUP:  ret=GAME_NEXT; break;
    case SDLK_PAGEDOWN:ret=GAME_PREV; break;
    case SDLK_0: case SDLK_1: Screen_Update(0,0,640,480); break;
    default:;
   }

  }else if (event.type==SDL_KEYUP){
   /*Some key was released. Which?*/
   switch (event.key.keysym.sym){
    case SDLK_LEFT:  case SDLK_KP4: keyc&=0xF-8; break;
    case SDLK_RIGHT: case SDLK_KP6: keyc&=0xF-2; break;
    case SDLK_UP:    case SDLK_KP8: keyc&=0xF-1; break;
    case SDLK_DOWN:  case SDLK_KP2: keyc&=0xF-4; break;
    case SDLK_KP7:   keyc&=0xF-(8|1); break;
    case SDLK_KP9:   keyc&=0xF-(1|2); break;
    case SDLK_KP5:   case SDLK_SPACE: keyc=0;    break;
    case SDLK_KP1:   keyc&=0xF-(8|4); break;
    case SDLK_KP3:   keyc&=0xF-(4|2); break;
    default:;
   }
   
  }else if (event.type==SDL_MOUSEMOTION){
   /*Update mouse coordinates & display cursor*/
   if ((scr=Screen_Lock())!=NULL){
    Blit32(&cursorbk,scr,0,0,mx-CURSOR_HX,my-CURSOR_HY,CURSOR_WIDTH,CURSOR_HEIGHT);
    t1=mx; t2=my;
    mx+=event.motion.xrel;
    my+=event.motion.yrel;
    if (mx<GAME_MXMIN) mx=GAME_MXMIN;
    else if (mx>GAME_MXMAX) mx=GAME_MXMAX;
    if (my<GAME_MYMIN) my=GAME_MYMIN;
    else if (my>GAME_MYMAX) my=GAME_MYMAX;
    Blit32(scr,&cursorbk,mx-CURSOR_HX,my-CURSOR_HY,0,0,CURSOR_WIDTH,CURSOR_HEIGHT);
    Intens_Out(cursor,0,NULL,game_colors[1],INTENS_T,mx-CURSOR_HX,my-CURSOR_HY,scr);
    Screen_Unlock();
    Screen_Update(mx-CURSOR_HX,my-CURSOR_HY,CURSOR_WIDTH,CURSOR_HEIGHT);
    Screen_Update(t1-CURSOR_HX,t2-CURSOR_HY,CURSOR_WIDTH,CURSOR_HEIGHT);
   }
   
  }else if (event.type==SDL_MOUSEBUTTONDOWN){
   if (my<160);
   else if (my<175){
    if ((mx>577) && (mx<577+16)) mouc|=1;
   }else if (my<190){
    if (mx<562);
    else if (mx<577) mouc|=1|8;
    else if (mx<592) mouc|=1;
    else if (mx<592+16) mouc|=1|2;
   }else if (my<205){
    if (mx<577) mouc|=8;
    else if (mx<592) mouc|=0xF;
    else mouc|=2;
   }else if (my<220){
    if (mx<562-8);
    else if (mx<577) mouc|=4|8;
    else if (mx<592) mouc|=4;
    else if (mx<592+16) mouc|=4|2;
   }else if (my<235){
    if ((mx>577) && (mx<577+16)) mouc|=4;
   }else if (my<240);
   
   else if (my<270) ret=GAME_DIED;
   else if (my<300) ret=GAME_NEXT;
   else if (my<330) ret=GAME_PREV;
   else if (my<360) mouc|=0xF;
   else ret=GAME_EXIT;
    
  }else if (event.type==SDL_QUIT) ret=GAME_HALT; /*Exit program*/  
  
  else if(event.type==SDL_ACTIVEEVENT){ /*Pause? Redraw?*/
   if (event.active.gain){ /*Reinitialize timer & redraw*/
    Screen_Update(0,0,640,480);
    Timer_Set(20);
   }else Timer_Set(0); /*Sleep*/
  }
 
  if (ret!=-1) break; /*Some exit condition happened*/
 }
 
 /*Restore mouse covered area. May be necessary if next time the module is not asked to redraw*/
 if ((scr=Screen_Lock())!=NULL){
  Blit32(&cursorbk,scr,0,0,mx-CURSOR_HX,my-CURSOR_HY,CURSOR_WIDTH,CURSOR_HEIGHT);
  Screen_Unlock();
  Screen_Update(mx-CURSOR_HX,my-CURSOR_HY,CURSOR_WIDTH,CURSOR_HEIGHT);
 }
 
 Timer_Set(oldtimer);
 
 return ret;
}
