/*
** geffects.c
**
** Graphic effects: various graphic stuff, mostly graphic effects
*/


#include "baseinc.h"
#include "geffects.h"



/*Prepares a tImage structure to be a "window" in an another one
**returns true if window could be made among filling dest, otherwise
**zero and NULL in dest->i*/
adint MakeWindow(tImage* src, sadint x, sadint y, sadint w, sadint h, tImage* dest)
{
 if (x<0){ w+=x; x=0; }
 if (y<0){ h+=y; y=0; }
 if ((x+w)>(src->x)) w=(src->x)-x;
 if ((y+h)>(src->y)) h=(src->y)-y;
 if ((w<=0) || (h<=0)){
  dest->i=NULL;
  dest->c=NULL;
  dest->x=0; dest->y=0; dest->p=0;
  return 0;
 }
 dest->x=w;
 dest->y=h;
 dest->p=src->p;
 if ((src->c)!=NULL){
  dest->i=src->i;
  dest->c=(src->c)+(src->p)*y+x;
 }else{
  dest->i=(src->i)+(src->p)*y+x;
  dest->c=NULL;
 }
 return 1;
}


/*Pixelizes an area
**fade also darkens it: 256: normal intensity; 0: black
**pix is the size of pixels<<8. Fractional sizes are handled this way
**appropriately.
**Note: much faster when using larger pixels!
*/
void Effect_Pixelize(adint x, adint y, adint w, adint h,
                     adint fade, adint pix, tImage *dest)
{
 adint j=0,i,bj,bi,t,oj,oi,sy,sx;
 dword ra,ga,ba,oc;
 dword* img;
 
 if (pix<256) pix=256; /*At least 1 pixel block width / height is needed*/
 
 while(j<(h<<8)){
  oj=j;
  j+=pix;
  if (j>(h<<8)) j=h<<8;
  sy=(j>>8)-(oj>>8); /*y size of box to create*/
  
  if (sy>1){
   i=0;
   while(i<(w<<8)){
    oi=i;
    i+=pix;
    if (i>(w<<8)) i=w<<8;
    sx=(i>>8)-(oi>>8); /*x size of the box to create*/
    
    /*if (sx>0){ Can not happen...*/
     t=sy*sx;
     ra=0; ga=0; ba=0;
     img=(dest->i)+(y+(oj>>8))*(dest->p)+x+(oi>>8);
     for(bj=0;bj<sy;bj++){
      for(bi=0;bi<sx;bi++){
       ra+=(img[bi]>>16)&0xFF; ga+=(img[bi]>>8)&0xFF; ba+=img[bi]&0xFF;
      }
      img+=(dest->p);
     }
     ra=(ra*fade)/t; ga=(ga*fade)/t; ba=(ba*fade)/t; oc=((ra & 0xFF00)<<8)+(ga & 0xFF00)+(ba>>8);
     img=(dest->i)+(y+(oj>>8))*(dest->p)+x+(oi>>8);
     for(bj=0;bj<sy;bj++){
      for(bi=0;bi<sx;bi++) img[bi]=oc;
      img+=(dest->p);
     }
    /*}*/
   }
  }else/* if (sy)*/{ /*Do it faster this case; sx can only be 0, 1, or 2*/
   i=0;
   img=(dest->i)+(y+(oj>>8))*(dest->p)+x;
   while(i<(w<<8)){
    oi=i;
    i+=pix;
    if (i>(w<<8)) i=w<<8;
    sx=(i>>8)-(oi>>8); /*x size of the box to create*/
    
    if (sx>1){
     ra= ((*img)>>16)&0xFF; ga= ((*img)>>8)&0xFF; ba= (*img)&0xFF;
     ra+=(img[1]>>16)&0xFF; ga+=(img[1]>>8)&0xFF; ba+=img[1]&0xFF;
     if (fade!=256){ oc=((((((ra & 0x1FE)<<15)+(ba>>1))*fade)>>8)& 0xFF00FF)+(((ga*fade) & 0x1FE00)>>1);}
     else{ oc=((ra & 0x1FE)<<15)+((ga & 0x1FE)<<7)+(ba>>1); }
     *img=oc; img[1]=oc;
    }else/* if (sx)*/{
     if (fade!=256) /*Only necessary to do anything this case*/
      *img=(((((*img) & 0xFF00FF)*fade) & 0xFF00FF00)+((((*img) & 0xFF00)*fade) & 0xFF0000))>>8;
    }
    img+=sx;    
   }
  } /*End of sy==1*/

 }

}



/*RandomMap
**Generates a random style 'map', may be used for background
**randomness also depends on what was on - screen before, so it may be better
**to use on filled screen
**xs and ys are for x and y maximal allowed color change <<8 (256 means it is
**possible that a col1 folows a col2 pixel)*/
sadint effect_randommapc=0x7FFF;
sadint effect_randommapr=0;
sadint effect_randommaprb=0;
void Effect_RandomMap(dword col1, dword col2, sadint xs, sadint ys, tImage* dest)
{
 sadint *buf;
 adint i,j;
 sadint ml=1,rp=effect_randommapr;
 dword *ti, t;

 /*Allocate a 1 line buffer (when fails, cleans to col1)*/
 if((buf=malloc(sizeof(sadint)*(dest->x)))==NULL){
  /*Clean and exit*/
  ti=dest->i;
  for (j=0;j<(dest->y);j++){
   for (i=0;i<(dest->x);i++) ti[i]=col1;
   ti+=dest->p;
  }
  return;
 }

 /*Fill the buffer*/
 for (i=0;i<(dest->x);i++){
  effect_randommapr=effect_randommapr+ml*((effect_randommaprb & 0x3F))*xs;
  effect_randommaprb=((effect_randommaprb-ys-i)|0x50500000)*((effect_randommaprb+xs+i)|0x0A0A0000);
  /*'Randomly' change direction*/
  if ((effect_randommaprb & 0xFF)==0) ml=((effect_randommaprb>>8)%9)-4;
  if (ml==0) if ((effect_randommaprb & 0xF)==0x6) ml=((effect_randommaprb>>8)%5)-2;
  /*High color values: more probability*/
  if      ((effect_randommapc> 0xAFFF) && (ml> 1)){ if ((effect_randommaprb & 0xF)==0x5) ml--; }
  else if ((effect_randommapc<=0x3FFF) && (ml<-1)){ if ((effect_randommaprb & 0xF)==0x7) ml++; }
  /*At the top: even more probability*/
  if      ((effect_randommapc> 0xEFFF) && (ml>0)){ if ((effect_randommaprb & 0x3)==0x1) ml--; }
  else if ((effect_randommapc<=0x0FFF) && (ml<0)){ if ((effect_randommaprb & 0x3)==0x2) ml++; }
  if     (ml<-4) ml=-4;
  else if(ml> 4) ml= 4;
  /*Add to color*/
  effect_randommapc+=effect_randommapr-rp;
  rp=effect_randommapr;
  if      (effect_randommapc>0x10000) effect_randommapc=0x10000;
  else if (effect_randommapc<0x00000) effect_randommapc=0x00000;
  /*Save*/
  buf[i]=effect_randommapc;
 }

 /*Go on with map generation!*/
 ti=dest->i;
 for (j=0;j<(dest->y);j++){
  /*Generate a proper first pixel*/
  effect_randommapc=*buf; /*Load prev. line's first*/
  for (i=0;i<(dest->x);i++){

   /*Just like above: generate a random next pixel*/
   effect_randommapr=effect_randommapr+ml*((effect_randommaprb & 0x3F))*xs;
   effect_randommaprb=((effect_randommaprb-ys-(i&j))|0x50500000)*((effect_randommaprb+xs+(i&j))|0x0A0A0000)+ti[i]; /*Also get some randomization from the original contents*/
   /*'Randomly' change direction*/
   if ((effect_randommaprb & 0xFF)==0) ml=((effect_randommaprb>>8)%9)-4;
   if (ml==0) if ((effect_randommaprb & 0xF)==0x6) ml=((effect_randommaprb>>8)%5)-2;
   /*High color values: more probability*/
   if      ((effect_randommapc> 0xAFFF) && (ml> 1)){ if ((effect_randommaprb & 0xF)==0x5) ml--; }
   else if ((effect_randommapc<=0x3FFF) && (ml<-1)){ if ((effect_randommaprb & 0xF)==0x7) ml++; }
   /*At the top: even more probability*/
   if      ((effect_randommapc> 0xEFFF) && (ml>0)){ if ((effect_randommaprb & 0x3)==0x1) ml--; }
   else if ((effect_randommapc<=0x0FFF) && (ml<0)){ if ((effect_randommaprb & 0x3)==0x2) ml++; }
   /*Add to color*/
   effect_randommapc+=effect_randommapr-rp;
   rp=effect_randommapr;
   if      (effect_randommapc>0x10000) effect_randommapc=0x10000;
   else if (effect_randommapc<0x00000) effect_randommapc=0x00000;

   /*But now also compare it to the pixel from the previous line*/
   if     (effect_randommapc>(buf[i]+(ys<<8))){ effect_randommapc=buf[i]+(ys<<8); if ((effect_randommaprb & 0x3)==0x2) ml--; }
   else if(effect_randommapc+(ys<<8) < buf[i]){ effect_randommapc=buf[i]-(ys<<8); if ((effect_randommaprb & 0x3)==0x1) ml++; }
   if(effect_randommapc<(buf[i]+(ys<<2))){ if ((effect_randommaprb & 0xF)==0x6) ml--; }
   if(effect_randommapc+(ys<<2) > buf[i]){ if ((effect_randommaprb & 0xF)==0xD) ml++; }
   if     (ml<-4) ml=-4;
   else if(ml> 4) ml= 4;
   
   /*Save*/
   buf[i]=effect_randommapc;

   /*Also mix colors & save output*/
   t=effect_randommapc>>8;
   ti[i]=( (((( col1    & 0x00FF00FF)*t)& 0xFF00FF00)+((( col2    & 0x00FF00FF)*(256-t))& 0xFF00FF00)) >>8) +
            ((((col1>>8)& 0x00FF00FF)*t)& 0xFF00FF00)+((((col2>>8)& 0x00FF00FF)*(256-t))& 0xFF00FF00);
  }
  ti+=dest->p;
 }
 
 free(buf);
}


/*Create a three - dimensional looking border by appliing colul on the upper -
**left corner, colbr on the bottom right. (Use transparent black & white for
**the traditional effect) Note that the given coordinates have to be on -
**dest: it does not handle partial off - dest stuff*/
void Effect_3DBorder(dword colul, dword colbr, sadint x, sadint y, adint w, adint h, tImage* dest)
{
 adint i, tp=dest->p, tpi;
 dword* ti;
 dword colult=colul>>24, colbrt=colbr>>24, colult2=255-colult, colbrt2=255-colbrt;
 
 /*Top line*/
 ti=(dest->i)+x+y*tp;
 tpi=0;
 for (i=0;i<w-1;i++)
  ti[i]=( (((ti[i]& 0x00FF00FF)*colult)& 0xFF00FF00)+(((colul & 0x00FF00FF)*colult2)& 0xFF00FF00) +
          (((ti[i]& 0x0000FF00)*colult)& 0x00FF0000)+(((colul & 0x0000FF00)*colult2)& 0x00FF0000) )>>8;
 /*Left line*/
 for (i=0;i<h-1;i++,tpi+=tp)
  ti[tpi]=( (((ti[tpi]& 0x00FF00FF)*colult)& 0xFF00FF00)+(((colul & 0x00FF00FF)*colult2)& 0xFF00FF00) +
            (((ti[tpi]& 0x0000FF00)*colult)& 0x00FF0000)+(((colul & 0x0000FF00)*colult2)& 0x00FF0000) )>>8;
 /*Bottom line*/
 ti+=(h-1)*tp+1;
 for (i=0;i<w-1;i++)
  ti[i]=( (((ti[i]& 0x00FF00FF)*colbrt)& 0xFF00FF00)+(((colbr & 0x00FF00FF)*colbrt2)& 0xFF00FF00) +
          (((ti[i]& 0x0000FF00)*colbrt)& 0x00FF0000)+(((colbr & 0x0000FF00)*colbrt2)& 0x00FF0000) )>>8;
 /*Right line*/
 ti=(dest->i)+x+w-1+(y+1)*tp;
 tpi=0;
 for (i=0;i<h-1;i++,tpi+=tp)
  ti[tpi]=( (((ti[tpi]& 0x00FF00FF)*colbrt)& 0xFF00FF00)+(((colbr & 0x00FF00FF)*colbrt2)& 0xFF00FF00) +
            (((ti[tpi]& 0x0000FF00)*colbrt)& 0x00FF0000)+(((colbr & 0x0000FF00)*colbrt2)& 0x00FF0000) )>>8;
}

/*Like the above, but may be also used for thicker borders*/
void Effect_3DBorderT(dword colul, dword colbr, adint s, sadint x, sadint y, adint w, adint h, tImage* dest)
{
 while (s>0){
  s--;
  Effect_3DBorder(colul,colbr,x+s,y+s,w-s-s,h-s-s,dest);
 } 
}
