/*
** intens.c
**
** intensity map handling on 32bit ARGB surfaces
*/



#include "baseinc.h"
#include "rot.h"
#include "intens.h"



/*Blits an intensity map on an ARGB surface
**color is the fg color for the map
**flags:
** INTENS_T   Keep original transparency data
**            (Without this, it will overwrite)
** The rotation flags also have to be injected here
**imap is in one byte xy header + data format
**Note: Does not test if valid
*/
dword intens_blit_pc=0;
dword intens_blit_buf[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
byte  intens_blit_a=0;
byte  intens_blit_abuf[16]={0xFF,0xEE,0xDD,0xCC,0xBB,0xAA,0x99,0x88,0x77,0x66,0x55,0x44,0x33,0x22,0x11,0x00};
byte  intens_blit_ac=0;
void Intens_Blit(byte* imap, dword color, adint flags, sadint x, sadint y, tImage* dest)
{
 adint sx=((*imap)>>4)+1,sy=((*imap)&0xF)+1;
 adint i,j,k,t;
 dword oc;
 dword *di=(dest->i)+y*(dest->p)+x;
 sadint addy=sy,addx=sx;
 dword colc=color&0xFFFFFF;
 byte  cola=color>>24;

 di+=Rot(flags,&addx,&addy,dest->p);
 addy-=addx*sx;
 imap++;

 if (cola!=intens_blit_a){
  intens_blit_a=cola;
  if (cola) for(k=0;k<16;k++) intens_blit_abuf[k]=255-(((k+(k<<4))*((adint)(255-cola)))/255);
  else for(k=0;k<16;k++) intens_blit_abuf[k]=255-(k+(k<<4));
 }
 
 if (flags & INTENS_T){
 
  if ((colc!=intens_blit_pc)||(intens_blit_ac!=intens_blit_a)){
   intens_blit_pc=colc;
   intens_blit_ac=intens_blit_a;
   for(k=0;k<16;k++){
    t=255-intens_blit_abuf[k];
    intens_blit_buf[k]=(((colc & 0xFF00FF)*t)&0xFF00FF00)+(((colc & 0xFF00)*t)&0xFF0000);
   }
  }
 
  for(j=0;j<sy;j++){
   i=0;
   while(i<sx){
    k=((*imap)&0xF0)>>4;
    t=intens_blit_abuf[k];
    oc=intens_blit_buf[k];
    k=((*imap)&0xF)+1;
    imap++;
    i+=k;
    while(k>0){
     *di=(( ((((*di)&0xFF00FF)*t)&0xFF00FF00)+
            ((((*di)&0xFF00)*t)&0xFF0000)+
         oc)>>8)+((*di)&0xFF000000);
     di+=addx; k--;
    }
   }
   di+=addy;
  }
  
  return;
 }

 for(j=0;j<sy;j++){
  i=0;
  while(i<sx){
   k=((*imap)&0xF)+1;
   oc=((intens_blit_abuf[((*imap)&0xF0)>>4])<<24)+(color & 0xFFFFFF);
   imap++;
   i+=k;
   while(k>0){ *di=oc; di+=addx; k--; }
  }
  di+=addy;
 }
}


/*Returns size of image which has both header bytes*/
adint Intens_Len(byte* imap)
{
 adint i=2,j,
       sx=(imap[1]>>4)+1,
       sy=(imap[1]&0xF)+1;
 if ((*imap)&1) return 2; /*Copied image*/
 while (sy){
  j=sx;
  while (j){
   j-=(imap[i]&0xF)+1;
   i++;
  }
  sy--;
 }
 return i;
}


/*Indexes an intensity map data
**This speeds up displaying
**imap is the traditional format intensity map, idx is the place to write
**the indexes to*/
void Intens_Index(byte* imap, adint* idx)
{
 adint i=1,ci=0;
 
 while (ci<=((adint)(*imap))){
  idx[ci]=i;
  ci++;
  i+=Intens_Len(imap+i);
 }
}


/*Outputs one from the intensity maps
**imap is the intensity map,
**ino is the image number to copy
**idx points to the index for the intensity map. Can be NULL (slower)
**color is the color to use when displaying (repeatedely using the same is faster!)
**flags:
** Rotation flags, and
** INTENS_T to preserve original's transparency (transparent put)
**x,y: destination upper - left corner
**dest: destination surface
**Note that partial intensity maps will NOT be drawn!
**Returns a width & height combination: height<<8 + width
*/
adint Intens_Out(byte* imap, adint ino, adint* idx,
                 dword color, adint flags,
		 sadint x, sadint y, tImage* dest)
{
 adint i,ci,sx,sy,rt;

 while(1){

  i=1;
  if (ino>((adint)(*imap))) return 0;

  /*Positioning on the image data*/
  if (idx!=NULL)
   i=idx[ino];
  else{
   ci=0;
   while (ci<ino){
    ci++;
    i+=Intens_Len(imap+i);
   }
  }
 
  /*Following copy requests, appliing transformations*/
  if (imap[i] & 2){ if (flags & MIR_X) flags&=0xFFFFFFFF-MIR_X; else flags|=MIR_X; }
  if (imap[i] & 4){ if (flags & MIR_Y) flags&=0xFFFFFFFF-MIR_Y; else flags|=MIR_Y; }
  if (imap[i] & 8){ if (flags & MIR_XY45) flags&=0xFFFFFFFF-MIR_XY45; else flags|=MIR_XY45; }

  if (imap[i] & 1) ino=imap[i+1];
  else break;
 }
 
 imap+=i+1; /*At size XY data*/
 
 /*Check if it is possible to copy it. If not, exit.*/
 if (flags & MIR_XY45){ sx=((*imap)&0xF)+1; sy=((*imap)>>4)+1; }
 else{ sy=((*imap)&0xF)+1; sx=((*imap)>>4)+1; }
 rt=(sy<<8)+sx;
 if (dest==NULL) return rt;
 if (x<0) return rt;
 if (y<0) return rt;
 if (x+sx>(dest->x)) return rt;
 if (y+sy>(dest->y)) return rt;
 
 /*Display!*/
 Intens_Blit(imap, color, flags, x, y, dest);
 return rt;
}
