/*
** intro.c
**
** Running Cheetah Software short intro sequence
*/


#include "baseinc.h"
#include "ppcres.h"
#include "timer.h"
#include "screen.h"
#include "iconvppc.h"
#include "intro.h"
#include "geffects.h"
#include "blit32.h"
#include <SDL/SDL.h>


#define INTRO_W 512
#define INTRO_H 192


/*Display the RCS logo in the middle of the screen*/
/*(x and y are for positioning away relative to the middle)*/
/*fade is to darken it. 256 is normal intensity, 0 is black*/
void Intro_Cheetah(sadint x, sadint y, adint fade, tImage *dest)
{
 PPC_Iconv_Disp(rcs_logo,
  ((sadint)(((dest->x)-RCS_LOGO_W)>>1))+x,
  ((sadint)(((dest->y)-RCS_LOGO_H)>>1))+y,
  fade,rcs_logo[0],dest);
}



/*Runs intro sequence, exit is possible by pressing a key
**Wait is the number of seconds to wait before starting (if used as program
**intro when setting the screen mode may cut some)*/
void Intro(adint wait)
{
 adint oldtimer=Timer_Get();
 SDL_Event event;
 adint stage=0, frame=0, frameo=0,
       x,y,poll=1;
 tImage* scr;

 if (Timer_Set(20)) return; /*(50FPS) Failed timer set*/

 /*Clear the area for the intro*/
 if ((scr=Screen_Lock())==NULL){ Timer_Set(oldtimer); return; }
 x=((scr->x)-INTRO_W)>>1;
 y=((scr->y)-INTRO_H)>>1;
 Clear32(x,y,INTRO_W,INTRO_H,0,scr);
 Screen_Unlock();
 Screen_Update(x,y,INTRO_W,INTRO_H); 

 Timer_Reset();
 
 while(1){ /*Intro main event loop*/
 
  if (poll) if (SDL_PollEvent(&event)==0) poll=0; /*Getting events until any is available*/
 
  if ((poll==0) && (frameo!=frame)){ /*Spare time: New frame may be processed!*/
   frameo=frame;
    
   if ((scr=Screen_Lock())!=NULL){
    
    switch (stage){
     case 0: /*Wait*/
      if (frame>(wait*50)){ stage++; frame=0; }
      break;
     case 1: /*Fade in*/
      if (frame>128){ Intro_Cheetah(0,0,256,scr); stage++; frame=0; break; }
      Intro_Cheetah(0,0,frame<<1,scr);
      break;
     case 2: /*Wait with the logo displayed*/
      if (frame>100){ stage++; frame=0; }
      break;
     case 3: /*Pixelized fade out*/
      if (frame>=256){ stage++; frame=0; break; }
      Clear32(x,y,INTRO_W,INTRO_H,0,scr);
      Intro_Cheetah(0,0,256,scr);
      Effect_Pixelize(x,y,INTRO_W,INTRO_H,256-frame,frame<<5,scr);
     default:;
    }
     
    Screen_Unlock();
    Screen_Update(x,y,INTRO_W,INTRO_H);
   }

  } /*End of frame processing*/
  
  if (poll==0){ /*No events... Wait for one!*/
   if (SDL_WaitEvent(&event)==0) break; /*Exit on event read fault*/
   poll=1; /*Next time again pump all what can be found*/
  }

  if ((event.type==SDL_KEYDOWN) && stage) break;  /*Exit on keypress*/
  if (event.type==SDL_QUIT) break; /*Exit as well*/
  if (stage>3) break; /*End of intro*/
  if (event.type==SDL_USEREVENT) if (event.user.code==TIMER_CODE) frame++;  /*Timer event: increase frame*/
  
 } /*End of event loop*/
 
 Timer_Set(oldtimer);
}
