/*
** kyeinter.c
**
** Kye interactions
** Note that Kye has to be the first member (0) of the object chain
** since it can do such special interactions which need it to be the
** first.
*/



#include "kyemapmn.h"
#include "baseinc.h"
#include "kyeintms.h"
#include "kyeispec.h"
#include "kyeinter.h"




adint kye_interact_frame=0;
extern adint kye_interact_rndfall; /*This is for simulating random falls when a 
                                     round thing hits another round one*/
			           /*Used for viruses as well*/
                                   /*(Got from kyeintms.c)*/


/*Tries to push something with msig: returns 1 if succeed (among pushing the
**said object), 0 otherwise (Note: returns 1 as well if no object is there.
**"Nothing" can be pushed around)*/
adint Kye_I_Push(sadint x, sadint y, adint msig)
{
 sadint id;
 tKyeObject o;
 adint asig=Kye_I_Alloc(x,y,msig);
 adint msignew;
 /*Diagonal pushes (Kye): do not allow Kye moving through surrounding obstacles*/
 if ( ((msig & 0x3)==0x3) && (asig & 0x12) ) return 0;
 if ( ((msig & 0x6)==0x6) && (asig & 0x50) ) return 0; 
 if ( ((msig & 0xC)==0xC) && (asig & 0x48) ) return 0;
 if ( ((msig & 0x9)==0x9) && (asig & 0x0A) ) return 0;
 /*Where is the location to push?*/
 if (msig & 1) y--;
 if (msig & 2) x++;
 if (msig & 4) y++;
 if (msig & 8) x--;
 /*Check the location*/
 if (Kye_I_Filled(x,y,0)==0) return 1;
 if ((id=Kye_MapID(x,y))==-1) return 0; /*It is some wall or such*/
 if (Kye_Obj(id,&o)) return 0; /*Not an object*/
 if (((o.p & 0x20)!=0x20) || ((o.m & 0x0800)==0x0800)) return 0; /*Not pushable or locked*/
 /*It allows, now check if the pushed object can travell to where it supposed to*/
 /*Do not allow pushing objects not being on block (they may miss interactions & ugly)*/
 if ((msig & 0x5) && ((o.y & 0xFF00)!=o.y)) return 0;
 if ((msig & 0xA) && ((o.x & 0xFF00)!=o.x)) return 0;
 if ((msignew=Kye_I_Move(x,y,msig | (o.s & 0x10),(o.m | (o.m>>4))&0xF,id))==0) return 0;
 /*Slow objects: check if both blocks can move which they occupy*/
 if ((o.y & 0xFF00)!=o.y){
  if ((msignew & 0xF)!=(Kye_I_Move(x, (o.y)>>8,   (msignew & 0xF)|(o.s & 0x10),0,id)&0xF)) return 0;
  if ((msignew & 0xF)!=(Kye_I_Move(x,((o.y)>>8)+1,(msignew & 0xF)|(o.s & 0x10),0,id)&0xF)) return 0;
 }
 if ((o.x & 0xFF00)!=o.x){
  if ((msignew & 0xF)!=(Kye_I_Move( (o.x)>>8,   y,(msignew & 0xF)|(o.s & 0x10),0,id)&0xF)) return 0;
  if ((msignew & 0xF)!=(Kye_I_Move(((o.x)>>8)+1,y,(msignew & 0xF)|(o.s & 0x10),0,id)&0xF)) return 0;
 }
 /*Slow objects in non - interactive frame: workaround (Otherwise these would
 **add as push to their velocity resulting in crappy stuff)*/
 if ((o.y & 0xFF00)!=o.y) msignew&=0xA;
 if ((o.x & 0xFF00)!=o.x) msignew&=0x5;
 /*Check if actually push could happen (Not going back + not zero -above- )*/
 if ((msig & 1) && (msignew & 4)) return 0;
 if ((msig & 2) && (msignew & 8)) return 0;
 if ((msig & 4) && (msignew & 1)) return 0;
 if ((msig & 8) && (msignew & 2)) return 0;
 /*Again, slow objects. As they occupy 2 blocks when moving, check needs to
 **be done on them - since otherwise they may be interpreted as "moved out of
 **the block" while they are moving in. This looks for "moving in", and that
 **case rejects (Zero test for msignew is for that it was zeroed to keep
 **native velocities having effect in this case)*/
 if ((msig & 0x5) && ((msignew & 0xF)==0) && ((o.x & 0xFF00)!=o.x)){
  if ((o.m & 0x80) && ((((o.x)>>8)+1)!=x)) return 0;
  if ((o.m & 0x20) && (( (o.x)>>8   )!=x)) return 0;
 }
 if ((msig & 0xA) && ((msignew & 0xF)==0) && ((o.y & 0xFF00)!=o.y)){
  if ((o.m & 0x10) && ((((o.y)>>8)+1)!=y)) return 0;
  if ((o.m & 0x40) && (( (o.y)>>8   )!=y)) return 0;
 }
 /*Now push can go, set up new movement signature for the pushed object*/
 o.m|=msignew & 0xF;
 if ((msignew & 0x10) && ((o.p & 0x3)==2)) o.p|=0x3; /*Slow move*/
 Kye_I_SetObject(id,&o);
 return 1;
}


/*Calculates interactions in a frame for every object. The frame is
**static: Kye moves one unit in 8 frames, sliders, rockies do so as
**well, movers move in 16 etc.
**Gets: Kye's movement request:
** bit 0-3: Moves Up / Left / Down / Right
**Returns: A flag set:
** bits:
** 0: Set in each 8th frame (when Kye is on a block)
** 1: When message is available (later - help boxes)
** 2: When message is cancelled (later)
** 3: Kye died
** 4: Set if a diamond was collected
*/
adint kye_interact_pkyex=0; /*For followers*/
adint kye_interact_pkyey=0;
adint Kye_Interact(adint move)
{
 tKyeObject obj;
 sadint nv,rt,id,x,y,t,ps;
 adint ret=0;
 
 Kye_ObjReset(); /*Make sure standing on the first object*/
 
 if(kye_interact_frame==0){ /*Interaction frame: setting movement flags*/
 
  if ((move & 0x5)==0x5) move&=0xA; /*Native velocity is excepted to be available without conflicts*/
  if ((move & 0xA)==0xA) move&=0x5; /*So Kye's direction have to be corrected if the keyboard was hit with a brick :p */
 
  ret|=0x1; /*Return flags: Movement frame processed*/

  /*First clean all movement flags (If the object is in interactive frame)*/
  while((id=Kye_NextObj(&obj))!=-1){
   if (((obj.x & 0xFF)==0) && ((obj.y & 0xFF)==0)) {
    obj.m=0;
    if ((obj.p & 0x3)==0x3) obj.p=(obj.p & 0xFC)+2; /*Restore fast speed*/
   }else obj.m&=0xFFF0; /*Clear only pushes. They are done fast speed!*/
   Kye_ObjSet(id,&obj);
  }

  /*Go on with interactions*/
  for (ps=0; ps<2; ps++){ /*Push layer: pushers come first*/

   while((id=Kye_NextObj(&obj))!=-1){ /*Get an object*/
 
    /*Random fall...*/
    kye_interact_rndfall=(kye_interact_rndfall+((obj.x)>>8)) & 0xFF;
   
    if ( ((obj.x & 0xFF)==0) && ((obj.y & 0xFF)==0) &&  /*In interaction frame*/
         ((obj.m & 0x0800)!=0x0800) &&  /*And not locked*/
 	 ( ((obj.p & 0x10) && ps) || (((obj.p & 0x10)==0) && (ps==0)) ) ){ /*Appropriate layer (pushers at last)*/

     /*Remove killed object*/
     if ((obj.k>=200) && (obj.m==0)){
      if (obj.t==KYE_O_KYE) ret|=0x8;           /*Kye: killed*/
      else if (obj.t==KYE_O_DIAMOND) ret|=0x10; /*Collected diamond*/
      Kye_FreeObj(id);
     }
     if (obj.k) continue; /*Zombie or killed: should not do anything anymore*/
   
     x=obj.x>>8;
     y=obj.y>>8; /*Needed many times...*/
   
     /*Gathering which directions the object wishes to move*/
     rt=(obj.m & 0xF); /*Maybe some other object set it already*/
     nv=0;             /*Movement sig. of native velocity*/

     /*Kye*/
     if (obj.t==KYE_O_KYE){
      nv|=move; /*Kye: special: Add given movement signature*/
      kye_interact_pkyex=x;
      kye_interact_pkyey=y;
     }else if(((obj.p & 0x3)>0) && ((obj.f>>4)==0) && ((obj.r & 0x3F)==0)){
      /*Objects having native velocity and that can be in effect*/
     
      switch ((obj.p>>2) & 0x3){
       case 0: /*Straight movers*/
        switch (obj.r){
         case 0:   nv|=1; break;
         case 64:  nv|=2; break;
         case 128: nv|=4; break;
         case 192: nv|=8; break;
         default:;
        }
        break;
       
       case 1: /*Random follow*/
        switch (kye_interact_rndfall & 0x7){
         case 0: case 2: case 7: /*Follow*/
 	  if (x<kye_interact_pkyex) nv|=2;
 	  else if (x>kye_interact_pkyex) nv|=8;
	  if (y<kye_interact_pkyey) nv|=4;
	  else if (y>kye_interact_pkyey) nv|=1;
	  break;
	 case 1: case 4: case 5: /*Random movement*/
	  nv|=(kye_interact_rndfall>>3)&0xF;
	  if ((nv & 0x5)==0x5) nv&=0xA;
          if ((nv & 0xA)==0xA) nv&=0x5;
         default:; /*Stand still*/
        }
        break;
      
       case 2: /*Follow*/
        if (x<kye_interact_pkyex) nv|=2;
        else if (x>kye_interact_pkyex) nv|=8;
        if (y<kye_interact_pkyey) nv|=4;
        else if (y>kye_interact_pkyey) nv|=1;
        break;
       
       default:;
      }
     }
    
     /*If something special has to be done to native velocity (magnets...)*/
     nv=Kye_IS_IFrameNV(id,&obj,nv);
   
     /*Movement flags set, see where the object may go*/
     t=(rt|nv)|(obj.s & 0x10);
     if (obj.t==KYE_O_KYE) t|=0x80;
     rt=Kye_I_Move(x,y,t,obj.m & 0xF,id);
     /*(Not involving native vel. in fixed as it must be 0 yet)*/
     (obj.m)|=rt&(0xF-nv);   /*Pushes*/
     (obj.m)|=(rt&nv)<<4;    /*Going by native velocity*/
     (obj.m)|=nv<<12;        /*New native vel. Mostly for movers: changes it when pushes!*/
     if ((rt & 0x10) && ((obj.p & 0x3)==2)) obj.p|=0x3; /*Slow move*/
  
     /*Special interactions*/
     ret|=Kye_IS_IFrame(id,&obj,nv);

     /*Try to push if pusher with the native velocity when blocked*/
     if ((obj.m & 0xF0)!=(nv<<4)) if (obj.p & 0x10)
      if (Kye_I_Push(x,y,nv)) obj.m|=((obj.m)>>8)&0xF0; /*If pushed, can go (unless zeroed)*/
   
     /*Done!*/
     Kye_I_SetObject(id,&obj);
 
    }
   }
  }
  
  /*Now correct movement: if both native velocity and push is present for a
  **particular direction, clear the native velocity (push is faster). This
  **may happen in various cases when objects interact*/
  while((id=Kye_NextObj(&obj))!=-1)
   if (((obj.x & 0xFF00)==obj.x) && ((obj.y & 0xFF00)==obj.y)) {
    obj.m=(obj.m & 0xFF0F)|(obj.m & ((0xF-(obj.m & 0xF))<<4));
    Kye_ObjSet(id,&obj);
   }
 
 } 
 

 /*Movement frames: objects just move how their flags are set*/
 while((id=Kye_NextObj(&obj))!=-1){ /*Get an object*/
 
  /*Check if it is "pushed" somewhere, and attempt
  **to calculate new coordinates*/
  nv=((obj.p)&0x3)<<4;
  if (nv==48) nv=16; /*Temporary slow move*/
  /*First look for all - object "pushed" flag (which not necessarily
  **means it was pushed, pulling also happens this way, or anything can
  **not described by native motion*/
  if (obj.m & 1) obj.y-=32;
  if (obj.m & 2) obj.x+=32;
  if (obj.m & 4) obj.y+=32;
  if (obj.m & 8) obj.x-=32;
  /*Native velocity*/
  if (obj.m & 0x10) obj.y-=nv;
  if (obj.m & 0x20) obj.x+=nv;
  if (obj.m & 0x40) obj.y+=nv;
  if (obj.m & 0x80) obj.x-=nv;
  /*Rotation if any*/
  if (obj.m & 0x200) obj.r=(obj.r+8)&0xFF;
  if (obj.m & 0x400) obj.r=(obj.r-8)&0xFF;
  /*Reverse direction if any*/
  if ((obj.m & 0x100) || (obj.f & 0xF0)){
   switch (obj.p & 0x3){
    case 1: case 3: obj.f+=0x10; break; /*Slow movers (It will wrap when appropriate: it is one byte!*/
    default: obj.f+=0x20;               /*Fast movers*/
   }
  }
  else obj.f&=0x0F; /*Clean reverse dir. info - it might be left there*/
  /*If killing started (be careful since locking (slow moving objects mostly)
  **may make it wrapping what is ugly*/
  if (obj.k){ if (obj.k<255-15) obj.k+=15; else obj.k=255; }
  
  /*Special interactions*/
  Kye_IS_Frame(id,&obj);

  /*Update positions*/
  Kye_ObjSet(id,&obj);
  
 }
 
 kye_interact_frame=(kye_interact_frame+1) & 0x7;
 
 return ret;
}


/*Returns 1 if interaction frame comes*/
adint Kye_InteractFrame()
{
 if (kye_interact_frame) return 0;
 return 1;
}


/*Resets interaction*/
void Kye_Interact_Reset()
{
 kye_interact_frame=0;
 kye_interact_rndfall=0;
}
