/*
** kyeintms.c
**
** Kye interactions
** Some misc. functions used by both special and base object interactions
*/



#include "kyemapmn.h"
#include "baseinc.h"
#include "kyeispec.h"
#include "kyeintms.h"


adint kye_interact_rndfall=0; /*This is for simulating random falls when a 
                                round thing hits another round one*/
			      /*Used for viruses as well*/


/*Returns 1 if a block is filled with anything*/
/*msig is the movement signature how the requesting object wishes to
**enter the checked block. Bit 7 specifies if the object is Kye, Bit
**6 if the door should be uncovered*/
adint Kye_I_Filled(sadint x, sadint y, adint msig)
{
 sadint bl=Kye_Block(x,y), id=Kye_MapID(x,y), pass=0;
 tKyeObject o;
 if (((bl & 0x70)==0) && (id==-1)) return 0;
 
 if (((bl & 0x70)==0x20) || ((bl & 0x70)==0x30)){ /*Doors*/
  if ((msig & 0x80)==0){
   if (bl & 0x4) return 1; /*Not passable at all for non - Kye*/
   switch (bl & 0x3){ /*Directions (pass: passed?)*/
    case 0:  if ((msig & 0xF)==1) pass=1; else pass=0; break;
    case 1:  if ((msig & 0xF)==2) pass=1; else pass=0; break;
    case 2:  if ((msig & 0xF)==4) pass=1; else pass=0; break;
    default: if ((msig & 0xF)==8) pass=1; else pass=0;
   }
  }else{ /*All - passable doors: not so strict*/
   switch (bl & 0x3){
    case 0:  if (msig & 0x5){ if (msig & 1) pass=1; else pass=0; } else return 0; break;
    case 1:  if (msig & 0xA){ if (msig & 2) pass=1; else pass=0; } else return 0; break;
    case 2:  if (msig & 0x5){ if (msig & 4) pass=1; else pass=0; } else return 0; break;
    default: if (msig & 0xA){ if (msig & 8) pass=1; else pass=0; } else return 0;
   }
  }
  /*Set the uncovered flag*/
  if (msig & 0x40) Kye_BlockSet(x,y,bl | 0x8);
  return 1-pass;
 }
 
 if (Kye_Obj(id,&o)) return 1;
 switch (msig & 0xF){
  case 1: if (((o.y & 0xFF)!=0) && ((o.m & 0x44)==0)) return 0; break; /*Slow object leaving*/
  case 2: if (((o.x & 0xFF)!=0) && ((o.m & 0x88)==0)) return 0; break;
  case 4: if (((o.y & 0xFF)!=0) && ((o.m & 0x11)==0)) return 0; break;
  case 8: if (((o.x & 0xFF)!=0) && ((o.m & 0x22)==0)) return 0; break;
  default:;
 }
 return 1;
}

/*Gets allocation signature around object. Bits are ordered this
**way:
**0 1 2
**3   4
**5 6 7
**from msig only bit 7 is used: Kye*/
adint Kye_I_Alloc(sadint x, sadint y, adint msig)
{
 adint ret=0;
 ret|=Kye_I_Filled(x-1,y-1,0/* | (msig & 0x80)*/);
 ret|=Kye_I_Filled(x,y-1,  1 | (msig & 0x80))<<1;
 ret|=Kye_I_Filled(x+1,y-1,0/* | (msig & 0x80)*/)<<2;
 ret|=Kye_I_Filled(x-1,y,  8 | (msig & 0x80))<<3;
 ret|=Kye_I_Filled(x+1,y,  0/* | (msig & 0x80)*/)<<4;
 ret|=Kye_I_Filled(x-1,y+1,4 | (msig & 0x80))<<5;
 ret|=Kye_I_Filled(x,y+1,  0/* | (msig & 0x80)*/)<<6;
 ret|=Kye_I_Filled(x+1,y+1,2 | (msig & 0x80))<<7;
 return ret;
}

/*Kye_I_Hit
**Returns object ID which the object would hit when moving with the given
**movement signature (-1 if no object)*/
sadint Kye_I_Hit(sadint x, sadint y, adint msig)
{
 if (msig & 1) y--;
 if (msig & 2) x++;
 if (msig & 4) y++;
 if (msig & 8) x--;
 return Kye_MapID(x,y);
}

/*Returns the edge signature of a block (topleft / topright /
**bottomleft / bottomright round? This returns either an object's or
**a map block's signature, depending on what is there*/
adint Kye_I_Edge(sadint x, sadint y)
{
 adint t=Kye_Block(x,y);
 tKyeObject o;
 if ((t & 0x70)!=0){
  switch (t & 0x70){
   case 0x10: return t & 0xF; /*Wall*/
   /* Doors behave as box - shaped */
   default: return 0;
  }
 }else{ /*Attempt to get object, then it's shape*/
  if (Kye_Obj(Kye_MapID(x,y),&o)==0){
   t=(o.s)<<(o.r>>6); /*Rotate the shape according to the object*/
   return ((t | (t>>4)) & 0xF);
  }
 }
 return 0;
}

/*Gets an x and y location and a movement signature & round flag,
**then returns a new movement signature how the object may move.
**(movr 0-3: movement sig., movr 4: round flag, 7: kye flag)
**movf is the movement the object already have: it is fixed, can not
**be altered
**In the return value bit 4 is set if the native velocity should be
**halved for a frame*/
adint Kye_I_Move(sadint x, sadint y, adint movr, adint movf, sadint id)
{
 adint ret=0,sig=Kye_I_Alloc(x,y,movr),t1;
 sadint i,mv,xa,ya,j;
 byte slides[32]={ /*First: edge sig, then alloc. sig, finally movement codes*/
  0x8,0x09,0x2,0x8, /*Going up, left*/
  0x4,0x14,0x8,0x2, /*Going up, right*/
  0x1,0x06,0x4,0x1, /*Going right, top*/
  0x8,0xC0,0x1,0x4, /*Going right, bottom*/
  0x1,0x28,0x2,0x8, /*Going down, left*/
  0x2,0x90,0x8,0x2, /*Going down, right*/
  0x2,0x03,0x4,0x1, /*Going left, top*/
  0x4,0x60,0x1,0x4};/*Going left, bottom*/
 /*Originally in the four movement handler these ifs were used for slides,
 **two in each, the data is extracted from these to reproduce them. This
 **way the program size became smaller
 ** if ((t1 & 0x8) && ((sig & 0x09)==0) && ((movr & 2)==0)) ret|=8 | 1 | 0x10; UP Left slide
 ** if ((t1 & 0x4) && ((sig & 0x14)==0) && ((movr & 8)==0)) ret|=2 | 1 | 0x10; UP Right slide
 ** if ((t1 & 0x1) && ((sig & 0x06)==0) && ((movr & 4)==0)) ret|=1 | 2 | 0x10; RIGHT Top slide
 ** if ((t1 & 0x8) && ((sig & 0xC0)==0) && ((movr & 1)==0)) ret|=4 | 2 | 0x10; RIGHT Bottom slide
 ** if ((t1 & 0x1) && ((sig & 0x28)==0) && ((movr & 2)==0)) ret|=8 | 4 | 0x10; DOWN Left slide
 ** if ((t1 & 0x2) && ((sig & 0x90)==0) && ((movr & 8)==0)) ret|=2 | 4 | 0x10; DOWN Right slide
 ** if ((t1 & 0x2) && ((sig & 0x03)==0) && ((movr & 4)==0)) ret|=1 | 8 | 0x10; LEFT Top slide
 ** if ((t1 & 0x4) && ((sig & 0x60)==0) && ((movr & 1)==0)) ret|=4 | 8 | 0x10; LEFT Bottom slide*/

 movr|=movf | 0x40; /*Also uncover doors*/
 /*First neutralize conflicting movements*/
 if ((movr & 0x5)==0x5) movr&=0xA;
 if ((movr & 0xA)==0xA) movr&=0x5;
 movr|=movf;
 if ((movr & 0xF)==0) return 0;
 
 /*Movement possibilities*/
 for (i=0;i<4;i++){
  mv=1<<i;
  switch (i){
   case 0:  xa=x;   ya=y-1; break;
   case 1:  xa=x+1; ya=y;   break;
   case 2:  xa=x;   ya=y+1; break;
   default: xa=x-1; ya=y;
  }
  if (movr & mv){
   if (movf & mv) ret|=mv; /*If the move already worked, leave it alone*/
   else if (Kye_I_Filled(xa,ya,movr & (0xF0 | mv))){ /*Something is there...*/
    if (((movr & (0xF-mv))==0) && (Kye_IS_Meet(id,Kye_MapID(xa,ya))!=0)) ret|=mv; /*Something interacts*/
    else if (movr & 0x10){ /*Round object: might bounce*/
     t1=Kye_I_Edge(xa,ya);
     for (j=0;j<8;j+=4) /*Slides: top/left + bottom/right*/
      if ( (t1 & ((adint)slides[(i<<3)+j])) &&
           ((sig & ((adint)slides[(i<<3)+j+1]))==0) &&
	   ((movr & ((adint)slides[(i<<3)+j+2]))==0) ) ret|=((adint)slides[(i<<3)+j+3]) | mv | 0x10;
    }
   }else ret|=mv;
  }
 }

 /*Finally: Diagonal moves. Not allow if there is something what blocks
 **diagonally. --Might-- have a bug, but i do not think so. If the above
 **returns such combination which would represent a diagonal move - but
 **currently i say that would be not even possible*/
 if (((ret & 0x3)==0x3) && (sig & 0x04) && (Kye_IS_Meet(id,Kye_MapID(x+1,y-1))==0)) return movf;
 if (((ret & 0x9)==0x9) && (sig & 0x01) && (Kye_IS_Meet(id,Kye_MapID(x-1,y-1))==0)) return movf;
 if (((ret & 0x6)==0x6) && (sig & 0x80) && (Kye_IS_Meet(id,Kye_MapID(x+1,y+1))==0)) return movf;
 if (((ret & 0xC)==0xC) && (sig & 0x20) && (Kye_IS_Meet(id,Kye_MapID(x-1,y+1))==0)) return movf;

 if ((ret&0x5)==0x5){ if (movf & 0x5) ret&=0xFA+(movf & 0x5); else ret&=0xFE -((kye_interact_rndfall&1)*3); }
 if ((ret&0xA)==0xA){ if (movf & 0xA) ret&=0xF5+(movf & 0xA); else ret&=0xFD -((kye_interact_rndfall&2)*3); }
 kye_interact_rndfall=(kye_interact_rndfall+sig)&0xFF;
 /*Random fall when both directions were set (kye_interact_rndfall provides
 **pseudo - randomness)*/
 
 return ret;
}


/*Kye_I_AllocBlks
**Allocates impassable blocks for an object to use for it's movement*/
void Kye_I_AllocBlks(sadint id, tKyeObject* o)
{
 sadint x=(o->x)>>8, y=(o->y)>>8, x2=(o->x+255)>>8, y2=(o->y+255)>>8;
 /*(Takes in account that half - speed objects can not be moved when are not
 ** on block - boundary in that direction)*/
 
 /*Pushing (full speed)*/
 /*Straight*/
 if ((o->m) & 0x1) { Kye_AllocBlock(x,y-1); Kye_AllocBlock(x2,y-1); }
 if ((o->m) & 0x2) { Kye_AllocBlock(x+1,y); Kye_AllocBlock(x+1,y2); }
 if ((o->m) & 0x4) { Kye_AllocBlock(x,y+1); Kye_AllocBlock(x2,y+1); }
 if ((o->m) & 0x8) { Kye_AllocBlock(x-1,y); Kye_AllocBlock(x-1,y2); }
 /*Diagonal: below with the diagonal movements*/
 
 /*Movement directions (Maybe half speed)*/
 /*Straight*/
 if (x==x2){
  if ((o->m) & 0x20) Kye_AllocBlock(x+1,y);
  if ((o->m) & 0x80) Kye_AllocBlock(x-1,y);
 }
 if (y==y2){
  if ((o->m) & 0x10) Kye_AllocBlock(x,y-1);
  if ((o->m) & 0x40) Kye_AllocBlock(x,y+1);
 }
 /*Diagonal*/
 if ( (x==x2) && (y==y2) ){
  if ((((o->m)|((o->m)>>4)) & 0x3)==0x3) Kye_AllocBlock(x+1,y-1);
  if ((((o->m)|((o->m)>>4)) & 0x6)==0x6) Kye_AllocBlock(x+1,y+1);
  if ((((o->m)|((o->m)>>4)) & 0xC)==0xC) Kye_AllocBlock(x-1,y+1);
  if ((((o->m)|((o->m)>>4)) & 0x9)==0x9) Kye_AllocBlock(x-1,y-1);
 }
}


/*Setting an object*/
void Kye_I_SetObject(sadint id, tKyeObject* o)
{
 Kye_IS_MeetAct(id,o); /*If some interaction happenned*/
 Kye_I_AllocBlks(id,o); /*Get blocks for moving*/
 Kye_ObjSet(id,o);
}

