/*
** kyeispec.c
**
** Kye interactions - object - specific specials
*/



#include "kyemapmn.h"
#include "baseinc.h"
#include "kyeintms.h"
#include "kyeispec.h"



/*Routines:
**Kye_IS_Meet(id,id)
**Returns 1 if, and only if it has sense to push two objects on each other
**as the said two objects have some interaction
**Kye_IS_MeetAct(id,*obj)
**From the object's propeties (movement) determines if some sort of running
**on each other happens, then sets up the receiver to act, and returns 1
**(also locks the movement of the one given)
**Kye_IS_IFrame(id,*obj,nv)
**Special object effects should be carried out in every interation
**(usually 8th) frame. nv is the currently calculated native velocity
**signature. Returns some propeties:
** 1: When message is available (later - help boxes)
** 2: When message is cancelled (later)
** 3: Kye died
** 4: Set if a diamond was collected
**    This case bit 8-11 will get the frame (object variant)
**(Others: 0)
**Kye_IS_Frame(id,*obj)
**Special effects should be carried out in every frame
*/




adint Kye_IS_Meet(sadint id1, sadint id2)
{
 tKyeObject o1,o2;
 if (Kye_Obj(id1,&o1)) return 0; /*Not existing*/
 if (Kye_Obj(id2,&o2)) return 0;

 if (
     ((o1.t==KYE_O_KYE) &&
      ( (o2.t==KYE_O_DIAMOND) || (o2.t==KYE_O_EBLOCK) || (o2.t==KYE_O_ERBLOCK) )
     ) || (
      ( (o1.t==KYE_O_DIAMOND) || (o1.t==KYE_O_EBLOCK) || (o1.t==KYE_O_ERBLOCK) )
      && (o2.t==KYE_O_KYE))
    ) return 1;

 if (((o1.t==KYE_O_BLACK) && (o1.r<64)) || /*Blacky can eat anything going on it!*/
     ((o2.t==KYE_O_BLACK) && (o2.r<64))) return 1;
 return 0;
}


adint Kye_IS_MeetAct(sadint id, tKyeObject* obj)
{
 adint i,ret=0;
 sadint rid, x=(obj->x)>>8, y=(obj->y)>>8;
 tKyeObject ro;
 
 for (i=0;i<8;i++){
 
  rid=-1;
  switch (i){
   case 0:  if ((obj->m) & 0x11) rid=Kye_MapID(x,y-1); break;
   case 1:  if ((((obj->m) | ((obj->m)>>4)) & 0x3)==0x3) rid=Kye_MapID(x+1,y-1); break;
   case 2:  if ((obj->m) & 0x22) rid=Kye_MapID(x+1,y); break;
   case 3:  if ((((obj->m) | ((obj->m)>>4)) & 0x6)==0x6) rid=Kye_MapID(x+1,y+1); break;
   case 4:  if ((obj->m) & 0x44) rid=Kye_MapID(x,y+1); break;
   case 5:  if ((((obj->m) | ((obj->m)>>4)) & 0xC)==0xC) rid=Kye_MapID(x-1,y+1); break;
   case 6:  if ((obj->m) & 0x88) rid=Kye_MapID(x-1,y); break;
   default: if ((((obj->m) | ((obj->m)>>4)) & 0x9)==0x9) rid=Kye_MapID(x-1,y-1);
  }
  
  if (Kye_Obj(rid,&ro)==0) if (((ro.m & 0xF0FF)==0) && (ro.k==0)){
   /*When not moving or being killed... Interacting stuff may be also pushed!*/
   switch (ro.t){

    case KYE_O_DIAMOND:
     if (obj->t==KYE_O_KYE){ ro.k=1; ret=1; Kye_ObjSet(rid,&ro); } /*Kye eating diamond*/
     break;
     
    case KYE_O_EBLOCK: case KYE_O_ERBLOCK:
     if (obj->t==KYE_O_KYE){ ro.k=1; Kye_ObjSet(rid,&ro); } /*Kye eating eatable blocks*/
     break;
     
    case KYE_O_BLACK: /*Go in eating stage & kill object*/
     obj->k=1; /*Kill*/
     ro.r=64;  /*Eat*/
     Kye_ObjSet(rid,&ro);
     break;
    
    default:;
   }
  
  }
 
 }
 
 if (ret) (obj->m)|=0x800; /*Lock*/
 return ret;
}


/*Returns true if a magnet is affecting the object in the given direction
**and distance. x and y are the object's coordinates. Returns movement
**signature like true value pointing to the direction where the magnet is*/
adint Kye_IS_MagAff(sadint x, sadint y, sadint xd, sadint yd)
{
 tKyeObject o;
 sadint msig=0;
 if ((xd==0) && (yd==0)) return 0;
 if ((xd!=0) && (yd!=0)) return 0;
 if ((xd<-2) || (yd<-2) || (xd>2) || (yd>2)) return 0;
 if (Kye_Obj(Kye_MapID(x+xd,y+yd),&o)==0) if (o.t==KYE_O_MAGNET){
  if ( (xd && (o.r & 0x40)) || (yd && ((o.r & 0x40)==0)) ){
   if (xd<0) msig=8;
   else if (xd>0) msig=2;
   else if (yd<0) msig=1;
   else msig=4;
   if ((xd>-2) && (xd<2) && (yd>-2) && (yd<2)) return msig;  /*1 distance: OK, magnet affects*/
   if ((Kye_I_Filled(x+(xd/2),y+(yd/2),msig)==0) || (Kye_MapID(x+(xd/2),y+(yd/2))==0xFFFE)) return msig; /*No object in the way:  affects*/
  }
 }
 return 0; 
}

/*Returns nonzero if a magnet is nearby. For farther magnets it returns a
**movement signature like direction, for ones next, signature<<4*/
adint Kye_IS_Magnet(sadint x, sadint y)
{
 sadint i,aff,sh;
 for (i=-2;i<=2;i++){
  if ((i==-1) || (i==1)) sh=4; else sh=0;
  if ((aff=Kye_IS_MagAff(x,y,i,0))!=0) return aff<<sh; /*(!=0 needed to prevent the compiler warning about... You know what :p)*/
  if ((aff=Kye_IS_MagAff(x,y,0,i))!=0) return aff<<sh;
 }
 return 0;
}

/*Pulls an object affected by a magnet towards it, also respecting the magnet's
**movement (If it is going away, then let the object follow). It gets the
**magnet's parameters*/
void Kye_IS_MagnetPull(tKyeObject *obj)
{
 tKyeObject o;
 sadint affx,affy,i,dir[2],id2,x=(obj->x)>>8,y=(obj->y)>>8,t;
 if ((obj->r) & 0x40){ affx=1; affy=0; dir[0]=8; dir[1]=2; } else { affx=0; affy=1; dir[0]=1; dir[1]=4; }
 /*Looks for objects next to the magnet*/
 for (i=-1;i<2;i+=2)
  if ((obj->m) & dir[(1-i)>>1])
   if (Kye_Obj(id2=Kye_MapID(x+affx*i,y+affy*i),&o)==0)
    if( ((o.t!=KYE_O_MAGNET) || (((o.r) & 0x40)==(0x40*affy))) &&
        (o.p & 0x20) && ((o.x & 0xFF)==0) && ((o.y & 0xFF)==0) ){
     t=dir[(1-i)>>1];
     if ((o.m & (((t>>2)+(t<<2)+(t<<6)) & 0xFF))==0) o.m|=t;
     /*There was a bug here: when an object oscillates between two magnets
     **it could happen that pulling away one changed the object's direction
     **so leaving an unpassable ghost block behind. So check if the opposite
     **direction was already "allocated"*/
     Kye_I_SetObject(id2,&o);
    }
 /*Looks for farther objects & attempt to pull them*/
 for (i=-2;i<3;i+=4)
  if (Kye_Obj(id2=Kye_MapID(x+affx*i,y+affy*i),&o)==0)
   if ( ((o.t!=KYE_O_MAGNET) || (((o.r) & 0x40)==(0x40*affy))) &&
        (o.p & 0x20) && ((o.x & 0xFF)==0) && ((o.y & 0xFF)==0) &&
	(Kye_I_Filled(x+affx*(i/2),y+affy*(i/2),0)==0) ){
    o.m|=(Kye_I_Move(x+affx*i,y+affy*i,dir[(2-i)>>2] | (o.s & 0x10), ((o.m)|((o.m)>>4)) & 0xF, id2) & 0xF);
    o.m&=0xFF0F;
    Kye_I_SetObject(id2,&o);
   }
}

/*Magnets: Follow Kye. Alters obj appropriately: the magnet's props*/
void Kye_IS_MagnetKyeF(tKyeObject *obj, tKyeObject *kye)
{
 sadint kx=(kye->x)>>8, ky=(kye->y)>>8, x=(obj->x)>>8, y=(obj->y)>>8;
 if ((obj->r) & 0x40){
  if (ky!=y) return;
  if ((kx==(x+1)) && ((kye->m) & 0x22) && (((obj->m) & 0xA)==0)){ obj->m|=2; return; }
  if ((kx==(x-1)) && ((kye->m) & 0x88) && (((obj->m) & 0xA)==0)){ obj->m|=8; return; }
  if ((kx==(x+2)) && (Kye_I_Filled(x+1,y,0)==0) && (((kye->m) & 0x88)==0)){ obj->m|=2; return; }
  if ((kx==(x-2)) && (Kye_I_Filled(x-1,y,0)==0) && (((kye->m) & 0x22)==0)){ obj->m|=8; return; }
 }else{
  if (kx!=x) return;
  if ((ky==(y+1)) && ((kye->m) & 0x44) && (((obj->m) & 0x5)==0)){ obj->m|=4; return; }
  if ((ky==(y-1)) && ((kye->m) & 0x11) && (((obj->m) & 0x5)==0)){ obj->m|=1; return; }
  if ((ky==(y+2)) && (Kye_I_Filled(x,y+1,0)==0) && (((kye->m) & 0x11)==0)){ obj->m|=4; return; }
  if ((ky==(y-2)) && (Kye_I_Filled(x,y-1,0)==0) && (((kye->m) & 0x44)==0)){ obj->m|=1; return; }
 }
}



adint Kye_IS_IFrameNV(sadint id, tKyeObject* obj, adint nv)
{
 if (((obj->t)==KYE_O_SLIDER) || ((obj->t)==KYE_O_ROCKY) || ((obj->t)==KYE_O_MOVER) || ((obj->t)==KYE_O_VIRUS))
  /*Lose native velocity when influenced by a magnet*/
  if (Kye_IS_Magnet((obj->x)>>8,(obj->y)>>8)) return 0;
 return nv; /*Otherwise*/
}


adint Kye_IS_IFrame(sadint id, tKyeObject* obj, adint nv)
{
 sadint id2,x=(obj->x)>>8,y=(obj->y)>>8,i;
 adint ret=0;
 tKyeObject obj2;

 switch (obj->t){
  case KYE_O_KYE:
   /*Diing is handled, so no need for that here...*/
   /*Kill when hitting viruses*/
   /*(Need to check if alive for situations for example when a virus goes in a
   ** blacky, and Kye is next to that blacky)*/
   if (Kye_Obj(Kye_MapID(x+1,y),&obj2)==0) if ((obj2.t==KYE_O_VIRUS) && (obj2.k==0)) obj->k=1;
   if (Kye_Obj(Kye_MapID(x,y+1),&obj2)==0) if ((obj2.t==KYE_O_VIRUS) && (obj2.k==0)) obj->k=1;
   if (Kye_Obj(Kye_MapID(x-1,y),&obj2)==0) if ((obj2.t==KYE_O_VIRUS) && (obj2.k==0)) obj->k=1;
   if (Kye_Obj(Kye_MapID(x,y-1),&obj2)==0) if ((obj2.t==KYE_O_VIRUS) && (obj2.k==0)) obj->k=1;
   /*Check for nearby magnets & pull them*/
   for (i=-2;i<=2;i++){
    if (Kye_IS_MagAff(x,y,i,0)){ Kye_Obj(id2=Kye_MapID(x+i,y),&obj2);
     Kye_IS_MagnetKyeF(&obj2,obj); Kye_I_SetObject(id2,&obj2); Kye_IS_MagnetPull(&obj2); }
    if (Kye_IS_MagAff(x,y,0,i)){ Kye_Obj(id2=Kye_MapID(x,y+i),&obj2);
     Kye_IS_MagnetKyeF(&obj2,obj); Kye_I_SetObject(id2,&obj2); Kye_IS_MagnetPull(&obj2); }
   }
   break;
   
  case KYE_O_SLIDER: case KYE_O_ROCKY:
   /*Turn the appropriate way when hitting a clocker*/
   if ((id2=Kye_I_Hit(x,y,nv))!=-1)
    if (Kye_Obj(id2,&obj2)==0) if (obj2.t==KYE_O_CLOCKER){ if (obj2.f & 1) (obj->m)|=0x400; else (obj->m)|=0x200; }
   /*--Solely for the purpose of oscillating magnet tricks--
   **Looks out if a magnet comes in the object's way, and gets itself pushed
   **when it does. Without this when timing is bad, the object may get missed
   **(passes over when the magnet goes to the opposing direction, so in
   ** neither frame can it affect) That damned trick worth this mess :lol:*/
   id2=((obj->m)>>4)&0xF; /*id2: Now just a temp variable*/
   switch (id2){
    case 1: case 4:
     switch (Kye_IS_Magnet(x,y-1+(((id2-1)*3)>>2))){ /*1 -> y-1; 4 -> y+1; trick to reduce size*/
      case 8: obj->m|=(Kye_I_Move(x,y,8 | ((obj->s) & 0x10), ((obj->m)|((obj->m)>>4)) & 0xF, id) & 0xF); break;
      case 2: obj->m|=(Kye_I_Move(x,y,2 | ((obj->s) & 0x10), ((obj->m)|((obj->m)>>4)) & 0xF, id) & 0xF); break;
      default:;
     }break;
    case 2: case 8:
     switch (Kye_IS_Magnet(x+1-((((id2-2)>>1)*3)>>2),y)){ /*2 -> x+1; 8 -> x-1; similar trick*/
      case 1: obj->m|=(Kye_I_Move(x,y,1 | ((obj->s) & 0x10), ((obj->m)|((obj->m)>>4)) & 0xF, id) & 0xF); break;
      case 4: obj->m|=(Kye_I_Move(x,y,4 | ((obj->s) & 0x10), ((obj->m)|((obj->m)>>4)) & 0xF, id) & 0xF); break;
      default:;
     }break;
    default:;
   }
   break;

  case KYE_O_ASLIDER: case KYE_O_AROCKY:
   /*Try to release new slider / rocky when necessary*/
   /*(Note: produced objects follow the producer)*/
   if (((obj->r) & 0x3F)==0){
    if (((obj->a) & 0x18)==0){ /*Release this case*/
     switch ((obj->r)>>6){
      case 0:  if (Kye_I_Filled(x,y-1,1)==0) Kye_NewObject(x,y,(obj->t)+1,0,0); break;
      case 1:  if (Kye_I_Filled(x+1,y,2)==0) Kye_NewObject(x,y,(obj->t)+1,1,0); break;
      case 2:  if (Kye_I_Filled(x,y+1,4)==0) Kye_NewObject(x,y,(obj->t)+1,2,0); break;
      default: if (Kye_I_Filled(x-1,y,8)==0) Kye_NewObject(x,y,(obj->t)+1,3,0);
     } /*(These new objects will be processed at the end thank to Kye_NextObj's behavior)*/
    }
    if ((obj->a) & 0x4){ /*Increase counter - release every third quarter rotation*/
     obj->a=((obj->a) & 0xFFE7)+1;
     if ((((obj->a) & 0x18)>>3)==3) (obj->a)&=0xFFE7;
    }
   }
   break;
     
  case KYE_O_MOVER:
   /*Blocked native velocity: reverse direction*/
   if (((obj->m) & 0xF0)!=(nv<<4)){ (obj->m)|=0x100; (obj->r)=((obj->r)+128)&0xFF; (obj->m)&=0x0FFF; }
   break;
   
  case KYE_O_TBLOCK:
   /*Kill if countdown finished*/
   if (((obj->r)==0) && ((obj->k)==0)) obj->k=1;
   break;
   
  case KYE_O_MAGNET:
   Kye_IS_MagnetPull(obj);
   break;

  default:;
 }

 return ret;
}



void Kye_IS_Frame(sadint id,tKyeObject *obj)
{

 switch (obj->t){
  case KYE_O_ASLIDER: case KYE_O_AROCKY: /*Rotate*/
   if (((obj->a) & 0xE0)==0){
    (obj->a)|=(8-(1<<((obj->a) & 0x3)))<<5;
    (obj->r)=((obj->r)+1) & 0xFF;
   }else (obj->a)-=0x20;
   break;
   
  case KYE_O_DIAMOND: /*Rotating animation*/
   (obj->r)++;
   break;
   
  case KYE_O_TBLOCK: /*Count down*/
   (obj->a)++;
   if ((obj->a)>4){
    obj->a=0;
    if (obj->r) (obj->r)--;
   }
   break;
  
  case KYE_O_BLACK: /*Rotate the animation sequence (also determines the eating sequence's end)*/
   (obj->r)++;
   if ((obj->r)==64) obj->r=0;
   break;
   
  case KYE_O_VIRUS: /*Animate them!*/
   switch ((obj->f) & 0xF){
    case 0: (obj->r)+=2; break; /*Blob*/
    case 1:                     /*Worm*/
    case 2: (obj->r)+=8; break; /*Gnasher*/
    case 3: (obj->r)+=4; break; /*Twister*/
    default:(obj->r)-=8;        /*Spike*/
   }
   break; 
  
  default:;
 }

}

