/*
** kyemap.c
**
** Kye playfield - related operations
*/


#include "baseinc.h"
#include "kyemap.h"




/*Structure for kye objects:
**
**typedef struct{
** word x;     Block location<<8
** word y;
** byte r;     Rotation
** byte f;     Frame 
** byte t;     Type
** byte i;     Internal flags (redraw, allocated)
** byte s;     Shape flags
** byte p;     Propeties flag
** word m;     Movement flags
** word a;     Other - object specific - attributes
** word d;     Interact with ID if the object can somehow interact with other
*/


byte* kye_map=NULL;
word* kye_mapo=NULL;
tKyeObject* kye_objects=NULL;
adint kye_objcur=0;
adint kye_seeknew=0;


#define KYE_WALL 0x10
/*Solid wall's type. Change on need. It is necessary for returning
**something appropriate for out-of-map areas*/


/*Kye_GetPTRs
**This function should not be used unless absolutely necessary (the screen
**updater). It returns the pointers for the map & objects.*/
void Kye_GetPTRs(byte** map, tKyeObject** objects)
{
 if (map!=NULL) *map=kye_map;
 if (objects!=NULL) *objects=kye_objects;
}

/*Allocates memory for the maps. If nonzero is returned, trouble!*/
/*May also be used to reset the map to empty, this case always returns 0*/
adint Kye_MapAlloc()
{
 adint i;
 if(kye_objects==NULL){
  if((kye_map=malloc((KYE_MX*KYE_MY)*sizeof(byte)))==NULL) return 1;
  if((kye_mapo=malloc((KYE_MX*KYE_MY)*sizeof(word)))==NULL){ free(kye_map); return 1; }
  if((kye_objects=malloc((KYE_OB)*sizeof(tKyeObject)))==NULL){ free(kye_map); free(kye_mapo); return 1; }
 }
 for (i=0;i<KYE_OB;i++) kye_objects[i].i=0; /*To make sure all allocated flags are clear*/
 for (i=0;i<(KYE_MX*KYE_MY);i++){ kye_mapo[i]=0xFFFF; kye_map[i]=0x80; } /*Update all*/
 return 0;
}

/*Frees the map memory*/
void Kye_MapFree()
{
 free(kye_map);
 free(kye_mapo);
 free(kye_objects);
 kye_objects=NULL; /*This one indicates the state of all*/
}


/*Internal: fill obj props*/
void Kye_ObjStructFill(adint id, tKyeObject* ret)
{
 ret->x=kye_objects[id].x;
 ret->y=kye_objects[id].y;
 ret->r=kye_objects[id].r;
 ret->f=kye_objects[id].f;
 ret->t=kye_objects[id].t;
 ret->s=kye_objects[id].s;
 ret->p=kye_objects[id].p;
 ret->m=kye_objects[id].m;
 ret->a=kye_objects[id].a;
 ret->k=kye_objects[id].k;
 ret->d=kye_objects[id].d;
 /*Do not return internals...*/
}


/*Returns the next object filled in ret. Returns -1 if all objects were
**returned (wraparound), otherwise the object's ID.
**New objects created during running this will be added to the end, and will
**be returned after the listing of the existed ones ends. Note that when it
**is already working on the new object list, it is unable to get any new
**object anymore!*/
adint Kye_NextObj(tKyeObject* ret)
{
 if (kye_seeknew==0){

  if (kye_objcur==KYE_OB){ kye_objcur=0; kye_seeknew=1; goto Kye_NextObjNew; }
  while(((kye_objects[kye_objcur].i) & 0x2)!=0x2){ /*Not allocated*/
   kye_objcur++;
   if (kye_objcur==KYE_OB){ kye_objcur=0; kye_seeknew=1; goto Kye_NextObjNew; }
  }
  Kye_ObjStructFill(kye_objcur,ret);
  (kye_objects[kye_objcur].i)&=0xFB; /*Clear "new" flag*/
  kye_objcur++; 
  return kye_objcur-1;
  
 }
 Kye_NextObjNew: /*Looking through new objects*/
 
 if (kye_objcur==KYE_OB){ kye_objcur=0; kye_seeknew=0; return -1; }
 while(((kye_objects[kye_objcur].i) & 0x6)!=0x6){ /*Not a new object*/
  kye_objcur++;
  if (kye_objcur==KYE_OB){ kye_objcur=0; kye_seeknew=0; return -1; }
 }
 Kye_ObjStructFill(kye_objcur,ret);
 (kye_objects[kye_objcur].i)&=0xFB; /*Clear "new" flag*/
 kye_objcur++; 
 return kye_objcur-1;
 
}
/*Resets the object counter (seeks to the first object)*/
void Kye_ObjReset()
{
 kye_objcur=0;
 kye_seeknew=0;
}

/*Returns object data selected by ID
**Does not alter the receiving values when the object does not exist, returns
**nonzero that case*/
adint Kye_Obj(adint id, tKyeObject* ret)
{
 if (id>=KYE_OB) return 1;
 if (((kye_objects[id].i)& 0x2)==0x2){
  Kye_ObjStructFill(id,ret);
  return 0;
 }
 return 1;
}


/*Internal: removes an ID form a map block*/
void Kye_IDClearPrim(adint x, adint y, adint id, adint up)
{
 adint moff=x+y*KYE_MX;
 if (kye_mapo[moff]==id) kye_mapo[moff]=0xFFFF; /*Move --in-- is necessary to clean FFFE*/
 else if(kye_mapo[moff]<KYE_OB) kye_objects[kye_mapo[moff]].i|=1; /*Update old occupier*/
 if (up) kye_map[moff]|=0x80;
 /*Check for nearby objects if they occupy this area except for the current one*/
 if (y>0) if (kye_mapo[moff-KYE_MX]<KYE_OB) if (kye_mapo[moff-KYE_MX]!=id)
  if (kye_objects[kye_mapo[moff-KYE_MX]].y>((y-1)<<8)){
   kye_mapo[moff]=kye_mapo[moff-KYE_MX];
   if (up) kye_objects[kye_mapo[moff-KYE_MX]].i|=1;
  }
 if (y<KYE_MY-1) if (kye_mapo[moff+KYE_MX]<KYE_OB) if (kye_mapo[moff+KYE_MX]!=id)
  if (kye_objects[kye_mapo[moff+KYE_MX]].y<((y+1)<<8)){
   kye_mapo[moff]=kye_mapo[moff+KYE_MX];
   if (up) kye_objects[kye_mapo[moff+KYE_MX]].i|=1;
  }
 if (x>0) if (kye_mapo[moff-1]<KYE_OB) if (kye_mapo[moff-1]!=id)
  if (kye_objects[kye_mapo[moff-1]].x>((x-1)<<8)){
   kye_mapo[moff]=kye_mapo[moff-1];
   if (up) kye_objects[kye_mapo[moff-1]].i|=1;
  }
 if (x<KYE_MX-1) if (kye_mapo[moff+1]<KYE_OB) if (kye_mapo[moff+1]!=id)
  if (kye_objects[kye_mapo[moff+1]].x<((x+1)<<8)){
   kye_mapo[moff]=kye_mapo[moff+1];
   if (up) kye_objects[kye_mapo[moff+1]].i|=1;
  }
 /*(Diagonals were not implemented: they were not necessary)*/
}
/*Internal: cleans an ID from the map*/
void Kye_IDClear(adint xb, adint yb, adint id, adint up)
{
 adint x=xb>>8, y=yb>>8;
                  Kye_IDClearPrim(x,y,id,up);
 if (xb!=(x<<8)){ Kye_IDClearPrim(x+1,y,id,up);
  if (yb!=(y<<8)) Kye_IDClearPrim(x+1,y+1,id,up); }
 if (yb!=(y<<8))  Kye_IDClearPrim(x,y+1,id,up);
}
/*Internal: adds an ID to a map block, does nothing if already added*/
void Kye_IDAddPrim(adint x, adint y, adint id, adint up)
{
 adint moff=x+y*KYE_MX;
 /*If an object is already there, set it's update flag to make sure it
 **will be updated if this was an updating request*/
 if (up && (kye_mapo[moff]<KYE_OB)) kye_objects[kye_mapo[moff]].i|=1;
 if (up) kye_map[moff]|=0x80;
 if ((kye_mapo[moff]<KYE_OB) && (kye_objects[id].k)) return;
 /*A diing object should not take over an alive one's place as it may
 **disappear leaving the block improperly empty if the alive one did not
 **move away during*/
 kye_mapo[moff]=id;
}
/*Internal: adds an ID to the map*/
void Kye_IDAdd(adint xb, adint yb, adint id, adint up)
{
 adint x=xb>>8, y=yb>>8;
                  Kye_IDAddPrim(x,y,id,up);
 if (xb!=(x<<8)){ Kye_IDAddPrim(x+1,y,id,up);
  if (yb!=(y<<8)) Kye_IDAddPrim(x+1,y+1,id,up); }
 if (yb!=(y<<8))  Kye_IDAddPrim(x,y+1,id,up);
}


/*Kye_AllocBlock
**Allocates a block for an object to where it will --surely-- move. So when
**an object decides to move, it has to! To prevent trouble, these blocks
**will be signed "KYE_IMPASSABLE" (0xFFFE) which any object may clear - so
**if used appropriately the object which will just move there now.*/
void Kye_AllocBlock(sadint xb, sadint yb)
{
 if ((xb>=KYE_MX) || (yb>=KYE_MY) || (xb<0) || (yb<0)) return;
 if (kye_mapo[xb+yb*KYE_MX]==0xFFFF) kye_mapo[xb+yb*KYE_MX]=0xFFFE;
}


/*Frees an object appropriately*/
void Kye_FreeObj(adint id)
{
 if (id>=KYE_OB) return;
 /*Removing it from the map & setting dirty flags*/
 Kye_IDClear(kye_objects[id].x,kye_objects[id].y,id,1);
 /*Now can be cleaned up*/
 kye_objects[id].i=0; /*Just turn off the occupied flag*/
}

/*Returns a free object slot (id), or -1 if no more*/
sadint Kye_GetFreeObj()
{
 adint i=0;
 while(((kye_objects[i].i) & 0x2)==0x2){
  i++;
  if (i==KYE_OB){ return -1; }
 }
 return i;
}


/*Set object propeties. Returns nonzero if not possible*/
adint Kye_ObjSet(adint id, tKyeObject* so)
{
 if (id>KYE_OB) return 1;
 if ((so->x)>((KYE_MX-1)<<8)) so->x=(KYE_MX-1)<<8;
 if ((so->y)>((KYE_MY-1)<<8)) so->y=(KYE_MY-1)<<8;

 /*Check if the current setup is identical to the old or not*/
 so->i=kye_objects[id].i;            /*If update flag was already set...*/
 if (((kye_objects[id].i) & 0x2)==0) /*Not existing: new, so draw*/
  (so->i)|=0x7; /*Set all "draw", "allocated" and "new" flags*/
 else /*Existing: check if modified*/
 if ( ((so->x)!=(kye_objects[id].x)) ||
      ((so->y)!=(kye_objects[id].y)) ||
      ((so->r)!=(kye_objects[id].r)) ||
      ((so->f)!=(kye_objects[id].f)) ||
      ((so->t)!=(kye_objects[id].t)) ||
      ((so->s)!=(kye_objects[id].s)) ||
      ((so->k)!=(kye_objects[id].k)) ){
  (so->i)|=0x3; 
  /*Cleans the current location*/
  Kye_IDClear(kye_objects[id].x,kye_objects[id].y,id,1);
 }
 /*Objects standing still might lose their block if some other moves over.
 **Look out for this case, and set the update flag when necessary*/
 else if (kye_mapo[((so->x)>>8)+((so->y)>>8)*KYE_MX]!=id) (so->i)|=0x3;

 /*Set the new propeties*/
 kye_objects[id].x=so->x;
 kye_objects[id].y=so->y;
 kye_objects[id].r=so->r;
 kye_objects[id].f=so->f;
 kye_objects[id].t=so->t;
 kye_objects[id].i=so->i;
 kye_objects[id].s=so->s;
 kye_objects[id].p=so->p;
 kye_objects[id].m=so->m;
 kye_objects[id].a=so->a;
 kye_objects[id].k=so->k;
 kye_objects[id].d=so->d;
  
 /*Update map positions where it will have effect*/
 if ((so->i & 0x2)==0x2) Kye_IDAdd(so->x,so->y,id,(so->i) & 0x1);
 return 0;
}



/*Map*/

/*Returns an object ID in the block, or -1 if nothing is there*/
sadint Kye_MapID(sadint x, sadint y)
{
 sadint moff=x+(y*KYE_MX);
 if ((x<KYE_MX) && (y<KYE_MY) && (x>=0) && (y>=0))
  if (kye_mapo[moff]!=0xFFFF) return kye_mapo[moff];
 return -1;
}

/*Returns the type of the block. For out - of - map, will return KYE_WALL,
**which stands for wall*/
byte Kye_Block(sadint x, sadint y)
{
 if ((x>=KYE_MX) || (y>=KYE_MY) || (x<0) || (y<0)) return KYE_WALL;
 return kye_map[x+(y*KYE_MX)];
}

/*Changes a block on the map. Sets update flag when necessary*/
void Kye_BlockSet(sadint x, sadint y, byte props)
{
 adint moff=x+(y*KYE_MX);
 if ((x>=KYE_MX) || (y>=KYE_MY) || (x<0) || (y<0)) return;
 if ((kye_map[moff]&0x7F)!=(props & 0x7F)) props|=0x80; else props&=0x7F;
 kye_map[moff]=props;
}
