/*
** kyemap.c
**
** Kye playfield - related operations
*/


#if !defined(KYEMAP_H)
#define KYEMAP_H


#include "baseinc.h"

#define KYE_MX 32
#define KYE_MY 24
#define KYE_OB 1024
/*Map sizes. It is possible to change by recompiling.
**Number of objects. Again may be changed by recompiling.
**Neither of those are strict limits
*/
#define KYE_IMPASSABLE 0xFFFE
/* "Impassable" object type: used when setting movements for objects*/


/*Structure for kye objects:*/
typedef struct{
 word x;    /* Block location<<8 */
 word y;
 byte r;    /* Rotation */
 byte f;    /* Frame */
 byte t;    /* Type */
 byte i;    /* Internal flags (redraw, allocated) */
 byte s;    /* Shape flags */
 byte p;    /* Propeties flag */
 word m;    /* Movement flags */
 byte a;    /* Other - object specific - attributes */
 byte k;    /* Kill object animation & counter */
 word d;    /* Interact with ID if the object can somehow interact with other */
}tKyeObject;


/*Kye_GetPTRs
**This function should not be used unless absolutely necessary (the screen
**updater). It returns the pointers for the map & objects.*/
void Kye_GetPTRs(byte** map, tKyeObject** objects);

/*Allocates memory for the maps. If nonzero is returned, trouble!*/
/*May also be used to reset the map to empty, this case always returns 0*/
adint Kye_MapAlloc();
/*Frees the map memory*/
void Kye_MapFree();

/*Objects*/

/*Returns the next object filled in ret. Returns -1 if all objects were
**returned (wraparound), otherwise the object's ID.
**New objects created during running this will be added to the end, and will
**be returned after the listing of the existed ones ends. Note that when it
**is already working on the new object list, it is unable to get any new
**object anymore!*/
adint Kye_NextObj(tKyeObject* ret);
/*Resets the object counter (seeks to the first object)*/
void Kye_ObjReset();
/*Returns object data selected by ID
**Does not alter the receiving values when the object does not exist, returns
**nonzero that case*/
adint Kye_Obj(adint id, tKyeObject* ret);
/*Frees an object appropriately*/
void Kye_FreeObj(adint id);
/*Returns a free object slot (id), or -1 if no more*/
sadint Kye_GetFreeObj();
/*Set object propeties. Returns nonzero if not possible*/
adint Kye_ObjSet(adint id, tKyeObject* so);
/*Kye_AllocBlock
**Allocates a block for an object to where it will --surely-- move. So when
**an object decides to move, it has to! To prevent trouble, these blocks
**will be signed "KYE_IMPASSABLE" (0xFFFE) which any object may clear - so
**if used appropriately the object which will just move there now.*/
void Kye_AllocBlock(sadint xb, sadint yb);

/*Map*/

/*Returns an object ID in the block, or -1 if nothing is there*/
sadint Kye_MapID(sadint x, sadint y);
/*Returns the type of the block. For out - of - map, will return KYE_WALL,
**which stands for wall*/
byte Kye_Block(sadint x, sadint y);
/*Changes a block on the map. Sets update flag when necessary*/
void Kye_BlockSet(sadint x, sadint y, byte props);

#endif
