/*
** kyemapup.c
**
** Map updater. Using the dirty flags of the map it updates the
** necessary screen areas
*/


#include "kyemapup.h"
#include "kyemap.h"
#include "screen.h"
#include "blit32.h"
#include "baseinc.h"



#define KYE_BLK_W 16
#define KYE_BLK_H 16


adint kye_scrx=0;
adint kye_scry=0;
void (**kye_objfunclist) (sadint,sadint,sadint,sadint,tImage*) =NULL;
void (**kye_mapfunclist) (sadint,sadint,sadint,sadint,tImage*) =NULL;
tImage* kye_bgimage=NULL;


/*Kye_ScrPos
**This sets where is the upper - left corner of the map on the screen*/
void Kye_ScrPos(adint x, adint y)
{
 kye_scrx=x;
 kye_scry=y;
}
/*Kye_SetBGImage
**sets background image to use. Must be called at least once before drawing
**maps!*/
void Kye_SetBGImage(tImage* bg)
{
 kye_bgimage=bg;
}

/*Kye_ObjFuncList
**Kye_MapFuncList
**Passes a pointer list to the functions need to be called to draw each
**object type. It has to have 256 elements, optionally with NULLs where
**necessary
**MapFuncList is the same for map blocks; there 8 functions are needed*/
void Kye_ObjFuncList(void (**funclist) (sadint,sadint,sadint,sadint,tImage*))
{
 kye_objfunclist=funclist;
}
void Kye_MapFuncList(void (**funclist) (sadint,sadint,sadint,sadint,tImage*))
{
 kye_mapfunclist=funclist;
}


/*Kye_ScrUp
**Updates the screen under dirty map blocks & clears dirty flags*/
void Kye_ScrUp()
{
 adint i,j,uno=0,t=0;
 byte* map;
 Kye_GetPTRs(&map,NULL);
 
 for (j=0;j<KYE_MY;j++){
  i=0;
  while(i<KYE_MX){
   while(1){ /*Find line to update*/
    if (map[i+t] & 0x80){ map[i+t]&=0x7F; uno++; } /*Clean dirty flag*/
    else if (uno!=0){
     if ((i+1)==KYE_MX) break;
     if (map[i+t+1] & 0x80) uno++; /*Only one gap between 2 updated? Then rather update too... (It appears to be costly to set an update region)*/
     else break;
    }
    i++;
    if (i==KYE_MX) break;
   }
   if (uno){ /*Update if necessary*/
    Screen_Update((i-uno)*KYE_BLK_W+kye_scrx, j*KYE_BLK_H+kye_scry, uno*KYE_BLK_W, KYE_BLK_H);
    uno=0;
   }
  }
  t+=KYE_MX;
 }
}

/*Kye_MapUp
**Redraws map areas where necessary. bgimg is the image to fill background
**with; it has to be supplied.
**does not alter flags as they are needed again at screen updating*/
void Kye_MapUp(tImage* screen, tImage* bgimg)
{
 adint i,j,t=0;
 byte* map;
 Kye_GetPTRs(&map,NULL);
 
 for (j=0;j<KYE_MY;j++){
  for (i=0;i<KYE_MX;i++){
   if (map[i+t]&0x80){
    Blit32(bgimg,screen, i*KYE_BLK_W,j*KYE_BLK_H, i*KYE_BLK_W+kye_scrx,j*KYE_BLK_H+kye_scry, KYE_BLK_W,KYE_BLK_H);
    if (kye_mapfunclist[(map[i+t]>>4)&0x7]!=NULL)
     kye_mapfunclist[(map[i+t]>>4)&0x7] (0, map[i+t], i*KYE_BLK_W+kye_scrx, j*KYE_BLK_H+kye_scry, screen);
   }
   /* Beta purposes: to display allocation on the entire screen (was used
   ** to hunt allocation related glitches. Note that when uncommented,
   ** objects may disappear: this is due to that it sets all map blocks
   ** to update while the object updater may not redraw objects over them
   ** if it does not "think" it necessary - based on the original behavior
   **  if (Kye_MapID(j,i)!=-1) screen->i[j*16+8+kye_scrx+(i*16+kye_scry+8)*(screen->p)]=0xFFFFFF;
   **  else screen->i[j*16+8+kye_scrx+(i*16+kye_scry+8)*(screen->p)]=0x000000;
   **  map[i+t]|=0x80;
   */
  }
  t+=KYE_MX;
 }
}

/*Kye_ObjUp
**Redraws objects if they were changed. It also clears the changed flags
**appropriately.*/
void Kye_ObjUp(tImage* screen)
{
 adint i,j;
 tKyeObject* obj;
 Kye_GetPTRs(NULL,&obj);
 
 for (j=0;j<4;j++){ /*Layer loop*/
  for (i=0;i<KYE_OB;i++){
   if (((obj[i].i) & 0x3)==0x3){ /*Allocated and changed*/
    if (j==((obj[i].p)>>6)){     /*Display layer*/
     if (kye_objfunclist[obj[i].t]!=NULL)
      kye_objfunclist[obj[i].t] (obj[i].r,obj[i].f | (obj[i].k<<8),
                                 ((((adint)(obj[i].x))*KYE_BLK_W)>>8)+kye_scrx,
				 ((((adint)(obj[i].y))*KYE_BLK_H)>>8)+kye_scry,
				 screen);
     (obj[i].i)&=0xFF -1; /*Turn off changed flag*/
    }
   }
  }
 }
}

/*Kye_Update
**Updates everything appropriately on the map, and displays them*/
void Kye_Update()
{
 tImage* scr;
 if ((scr=Screen_Lock())==NULL) return;
 Kye_MapUp(scr,kye_bgimage);
 Kye_ObjUp(scr);
 Screen_Unlock();
 Kye_ScrUp();
}
