/*
** kyeobjs.c
**
** Functions to draw each object in kye
*/


#include "kyeobjs.h"
#include "baseinc.h"
#include "intens.h"
#include "blit32.h"
#include "kyeobase.h"



/*Sets a color used in Kye.
**These are:
** 0: outer bound   (0x555555)   (black)
** 1: kye color     (0x44EE22)   (green)
** 2: base shape c. (0xEEDD11)   (yellow)
** 3: arrows        (0xEE5522)   (red)
** 4: auto slider   (0x2222EE)   (blue)
** 5: walls         (0x888888)   (gray)
** 6: black hole    (0x111111)   (black)
** 7: blob          (0xEE11DD)   (purple)
** 8: viruses       (0x779977)   (black)
** 9: diamond, 1    (0x334455EE) (blue)
**10: diamond, 2    (0x339999EE) (bright blue)
**11: background    (0x000000)   (white)
**12: Kye door      (0x66EE3311) (red)
**13: All ob. door  (0x775555EE)
**14: Uncover arrow (0x8833EE33)
*/
void Kye_Color(byte i, dword color)
{
 kye_colors[i]=color;
}

/*Frees indexes. Must be called before exitting to prevent leaks*/
void Kye_OB_Free()
{
 free(kye_index_s);
 free(kye_mixtmp[0].i);
 kye_index_s=NULL;
 kye_index_i=NULL;
 kye_index_o=NULL;
 kye_index_b=NULL;
 kye_mixtmp[1].i=NULL;
}

/*Sets up index tables & mixer
**(Without index tables the game can work, but the mixer is necessary)
**returns nonzero if mixer could not be allocated*/
adint Kye_OB_Alloc()
{
 /*Try to allocate some memory for mixing images*/
 if (kye_mixtmp[0].i==NULL) kye_mixtmp[0].i=malloc(512*sizeof(dword));
 if (kye_mixtmp[0].i!=NULL) kye_mixtmp[1].i=(kye_mixtmp[0].i)+256;
 else return 1;
 if(kye_index_s==NULL) kye_index_s=malloc(1024*sizeof(adint));
 if(kye_index_s!=NULL){
  kye_index_i=kye_index_s+256;
  kye_index_o=kye_index_s+512;
  kye_index_b=kye_index_s+768;
  Intens_Index(kye_shap,kye_index_s);
  Intens_Index(kye_in,kye_index_i);
  Intens_Index(kye_out,kye_index_o);
  Intens_Index(kye_big,kye_index_b);
 }
 return 0;
}


/*Kye
**neither rot and frame does anything*/
void Kye_O_Kye(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 Kye_OB_Base(x, y, dest, 0, Kye_OB_AddAlpha(kye_colors[1],(frame & 0xFF00)>>8), Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8));
}


/*Slider
**frame does not do anything*/
void Kye_O_Slider(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 Kye_OB_SR(rot, frame, x, y, dest, 0, 1, kye_colors[3],(frame & 0xFF00)>>8);
}

/*Rocky
**frame does not do anything*/
void Kye_O_Rocky(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 Kye_OB_SR(rot, frame, x, y, dest, 0, 0, kye_colors[3],(frame & 0xFF00)>>8);
}

/*Auto - Slider
**frame does not do anything*/
void Kye_O_ASlider(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 Kye_OB_SR(rot, frame, x, y, dest, 7, 2, kye_colors[4],(frame & 0xFF00)>>8);
}

/*Auto - Rocky
**frame does not do anything*/
void Kye_O_ARocky(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 Kye_OB_SR(rot, frame, x, y, dest, 7, 2, kye_colors[3],(frame & 0xFF00)>>8);
}

/*Clocker
**frame specifies if anticlocker: 0 for clocker, else for anticlocker
**rot does nothing*/
void Kye_O_Clocker(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 adint flags=INTENS_T;
 if (frame==0) flags|=MIR_Y;
 Kye_OB_Base(x, y, dest, 1, Kye_OB_AddAlpha(kye_colors[2],(frame & 0xFF00)>>8), Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8));
 Intens_Out(kye_shap,11,kye_index_s,Kye_OB_AddAlpha(kye_colors[3],(frame & 0xFF00)>>8),flags,x+2,y+2,dest);
}

/*Block
**Neither frame or rot does anything*/
void Kye_O_Block(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 Kye_OB_Base(x, y, dest, 1, Kye_OB_AddAlpha(kye_colors[2],(frame & 0xFF00)>>8), Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8));
}

/*RBlock
**Neither frame or rot does anything*/
void Kye_O_RBlock(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 Kye_OB_Base(x, y, dest, 0, Kye_OB_AddAlpha(kye_colors[2],(frame & 0xFF00)>>8), Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8));
}

/*EBlock (Eatable block)
**Neither frame or rot does anything*/
void Kye_O_EBlock(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 Kye_OB_Base(x, y, dest, 1,
  Kye_OB_AddAlpha(Kye_OB_AddAlpha(
   Kye_OB_MixColor(kye_colors[2],(0xFFFFFF-(kye_colors[0] & 0xFFFFFF))+(kye_colors[0] & 0xFF000000),192),
   (frame & 0xFF00)>>8), 96),
  Kye_OB_AddAlpha(Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8),144));
}

/*ERBlock
**Neither frame or rot does anything*/
void Kye_O_ERBlock(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 Kye_OB_Base(x, y, dest, 0,
  Kye_OB_AddAlpha(Kye_OB_AddAlpha(
   Kye_OB_MixColor(kye_colors[2],(0xFFFFFF-(kye_colors[0] & 0xFFFFFF))+(kye_colors[0] & 0xFF000000),192),
   (frame & 0xFF00)>>8), 96),
  Kye_OB_AddAlpha(Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8),144));
}

/*Wall
**frame is a bitmask of the rounded corners in order:
**up left, up right, bottom right, bottom left*/
void Kye_O_Wall(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 Intens_Out(kye_out,4-((frame&1)<<2),kye_index_o,kye_colors[5],INTENS_T,x,y,dest);
 Intens_Out(kye_out,4-((frame&2)<<1),kye_index_o,kye_colors[5],INTENS_T | MIR_Y,x+8,y,dest);
 Intens_Out(kye_out,4-((frame&4)   ),kye_index_o,kye_colors[5],INTENS_T | MIR_Y | MIR_X,x+8,y+8,dest);
 Intens_Out(kye_out,4-((frame&8)>>1),kye_index_o,kye_colors[5],INTENS_T | MIR_X,x,y+8,dest);
 Intens_Out(kye_shap,12,kye_index_s,0x99FFFFFF,INTENS_T,x+2,y+2,dest);
 Intens_Out(kye_shap,12,kye_index_s,0x99000000,INTENS_T | MIR_Y | MIR_X,x+2,y+2,dest);
}


/*MoverBase
**Draws basement for the mover
**Giving 0xFFFF for x asks for mixer drawing*/
void Kye_OB_MoverBase(adint basement, dword bcol, dword col, sadint x, sadint y, tImage* dest)
{
 adint flags=INTENS_T;
 if (x==0xFFFF){ x=0; y=0; flags=0; }
 Intens_Out(kye_out,1+((basement>>12)    ),kye_index_o,bcol,flags,x,y,dest);
 Intens_Out(kye_out,1+((basement>>8 )&0xF),kye_index_o,bcol,flags | MIR_Y,x+8,y,dest);
 Intens_Out(kye_out,1+((basement>>4 )&0xF),kye_index_o,bcol,flags | MIR_Y | MIR_X,x+8,y+8,dest);
 Intens_Out(kye_out,1+((basement    )&0xF),kye_index_o,bcol,flags | MIR_X,x,y+8,dest);
 Intens_Out(kye_in, 1+((basement>>12)    ),kye_index_i,col,INTENS_T,x+2,y+2,dest);
 Intens_Out(kye_in, 1+((basement>>8 )&0xF),kye_index_i,col,INTENS_T | MIR_Y,x+8,y+2,dest);
 Intens_Out(kye_in, 1+((basement>>4 )&0xF),kye_index_i,col,INTENS_T | MIR_Y | MIR_X,x+8,y+8,dest);
 Intens_Out(kye_in, 1+((basement    )&0xF),kye_index_i,col,INTENS_T | MIR_X,x+2,y+8,dest);
}

/*Mover
**frame: top 4 bits: reverse direction animation*/
void Kye_O_Mover(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 adint ino[2],flags[2],basement=0,mix,t,i,tf=frame&0xF0;
 dword ycol=Kye_OB_AddAlpha(kye_colors[2],(frame & 0xFF00)>>8),
       bcol=Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8),
       acol=Kye_OB_AddAlpha(kye_colors[3],(frame & 0xFF00)>>8);
 word rotb[8]={0x2200,0x1210,0x0220,0x0121,0x0022,0x1012,0x2002,0x2101};
 word turn[5]={0x0022,0x1122,0x1111,0x2211,0x2200};
 
 rot&=0xFF;

 Kye_OB_NewMix();
 if (tf==0){
  /*Set up basement for rotation (mix it seperatedely)*/
  t=Kye_OB_LinearSelect(rot,256,8,&mix);
  Kye_OB_MoverBase(rotb[t],bcol,ycol,0xFFFF,0xFFFF,&kye_mixtmp[0]);
  if (mix) Kye_OB_MoverBase(rotb[(t+1)&7],bcol,ycol,0xFFFF,0xFFFF,&kye_mixtmp[1]);
 }else{
  /*Set up for turning around*/
  t=Kye_OB_LinearSelect(tf,256,4,&mix);
  for (i=0;i<2;i++){
   basement=turn[t+i];
   switch(rot & 0xC0){
    case 0x00: break;
    case 0x40: basement=((basement>>4)+(basement<<12)) & 0xFFFF; break;
    case 0x80: basement=((basement>>8)+(basement<<8))  & 0xFFFF; break;
    default:   basement=((basement>>12)+(basement<<4)) & 0xFFFF; break;
   }
   Kye_OB_MoverBase(basement,bcol,ycol,0xFFFF,0xFFFF,&kye_mixtmp[i]);
  }
 }
 Mix32(&kye_mixtmp[0],&kye_mixtmp[1],mix);
 Blit32(&kye_mixtmp[1],dest,0,0,x,y,16,16);
 
 Kye_OB_NewMix();
 if (tf==0){
  /*Draw arrow for rotation*/
  Kye_OB_MixRot(&ino[0],&flags[0],&ino[1],&flags[1],&mix,rot);
  for (i=0;i<2;i++) Intens_Out(kye_shap,ino[i]+0,kye_index_s,acol,flags[i] & (0xFFFFFFFF-INTENS_T),2,2,&kye_mixtmp[i]);
 }else{
  /*Draw arrow for turning around*/
  /*(Sequence: 0,4,5,6,5,4,(0)*/
  t=Kye_OB_LinearSelect(tf,256,6,&mix);
  for (i=0;i<2;i++){
   ino[i]=(t+i);
   flags[i]=0;
   switch(ino[i]){
    case 0:  break;
    case 1:  ino[i]=4; break;
    case 2:  ino[i]=5; break;
    case 3:  ino[i]=6; break;
    case 4:  ino[i]=5; flags[i]=MIR_X; break;
    case 5:  ino[i]=4; flags[i]=MIR_X; break;
    default: ino[i]=0; flags[i]=MIR_X;
   }
   switch(rot & 0xC0){
    case 0x00: if (flags[i] & MIR_X) flags[i]&=0xFFFFFFFF-MIR_X; else flags[i]|=MIR_X; break; /*Damn, why C can not have a XOR!!*/
    case 0x40: flags[i]|=MIR_XY45; break;
    case 0x80: break;
    default:   flags[i]|=MIR_XY45 | MIR_Y; break;
   }
   Intens_Out(kye_shap,ino[i],kye_index_s,acol,flags[i],2,2,&kye_mixtmp[i]);
  }
 }
 Mix32(&kye_mixtmp[0],&kye_mixtmp[1],mix);
 Blit32(&kye_mixtmp[1],dest,0,0,x,y,16,16);
}

/*Diamond
**rot chooses which sides should "shine" (0-255)*/
void Kye_O_Diamond(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 dword dc1=Kye_OB_AddAlpha(kye_colors[9], (frame & 0xFF00)>>8),
       dc2=Kye_OB_AddAlpha(kye_colors[10],(frame & 0xFF00)>>8);
 dword c1, c2, c3, c4;
 rot&=0xFF;
 if (rot<128){ c1=Kye_OB_MixColor(dc1,dc2,rot<<1);
               c3=Kye_OB_MixColor(dc2,dc1,rot<<1);
 }else{        c3=Kye_OB_MixColor(dc1,dc2,(rot & 0x7F)<<1);
               c1=Kye_OB_MixColor(dc2,dc1,(rot & 0x7F)<<1);
 }
 rot=(rot+64)&0xFF;
 if (rot<128){ c2=Kye_OB_MixColor(dc1,dc2,rot<<1);
               c4=Kye_OB_MixColor(dc2,dc1,rot<<1);
 }else{        c4=Kye_OB_MixColor(dc1,dc2,(rot & 0x7F)<<1);
               c2=Kye_OB_MixColor(dc2,dc1,(rot & 0x7F)<<1);
 }
 Intens_Out(kye_in,3,kye_index_i,c1,INTENS_T,x+2,y+2,dest);
 Intens_Out(kye_in,3,kye_index_i,c2,INTENS_T | MIR_Y,x+8,y+2,dest);
 Intens_Out(kye_in,3,kye_index_i,c3,INTENS_T | MIR_Y | MIR_X,x+8,y+8,dest);
 Intens_Out(kye_in,3,kye_index_i,c4,INTENS_T | MIR_X,x+2,y+8,dest);
}

/*TBlock (Timer block)
**frame does not do anything, rot is for the countdown (255: full, 0: counted down)*/
void Kye_O_TBlock(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 dword c=Kye_OB_AddAlpha(kye_colors[3],(frame & 0xFF00)>>8);
 adint mix,t=Kye_OB_LinearSelect(rot&0xFF,256,24,&mix),i,j,xm,ym,flags,ino;
  
 Kye_OB_MixBase(1, Kye_OB_AddAlpha(kye_colors[2],(frame & 0xFF00)>>8), Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8));
 
 for(j=0;j<2;j++){ /*The two mixer images*/
  for(i=0;i<4;i++){ /*The four quarters*/
   if (t>(i*6)){
    if (t<((i+1)*6)) ino=6-(t%6)+6; /*A fraction has to be drawn*/
    else ino=6; /*Full*/
    switch(i){
     case 3:  flags=MIR_Y; xm=8; ym=2; break;
     case 2:  flags=MIR_Y | MIR_X | MIR_XY45; xm=8; ym=8; break;
     case 1:  flags=MIR_X; xm=2; ym=8; break;
     default: flags=MIR_XY45; xm=2; ym=2;
    }
    Intens_Out(kye_in,ino,kye_index_i,c,INTENS_T | flags,xm,ym,&kye_mixtmp[j]);
   } /*else: Empty*/
  }
  t++;
 }
 
 Mix32(&kye_mixtmp[0],&kye_mixtmp[1],mix);
 Blit32(&kye_mixtmp[1],dest,0,0,x,y,16,16);
}

/*Magnet
**rot specifies the direction: 0: up - down; 64: left - right etc.*/
void Kye_O_Magnet(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 dword col= Kye_OB_AddAlpha(kye_colors[3],(frame & 0xFF00)>>8),
       bcol=Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8);
 rot&=0x7F;
 rot>>=6;
 if (rot){
  Intens_Out(kye_out,5,kye_index_o,bcol,INTENS_T | MIR_XY45,x,y,dest);
  Intens_Out(kye_out,5,kye_index_o,bcol,INTENS_T | MIR_XY45 | MIR_Y,x+8,y,dest);
  Intens_Out(kye_out,5,kye_index_o,bcol,INTENS_T | MIR_XY45 | MIR_X,x,y+8,dest);
  Intens_Out(kye_out,5,kye_index_o,bcol,INTENS_T | MIR_XY45 | MIR_Y | MIR_X,x+8,y+8,dest);
  Intens_Out(kye_in,5,kye_index_i,col,INTENS_T | MIR_XY45, x+2,y+2,dest);
  Intens_Out(kye_in,5,kye_index_i,col,INTENS_T | MIR_XY45 | MIR_Y,x+8,y+2,dest);
  Intens_Out(kye_in,5,kye_index_i,col,INTENS_T | MIR_XY45 | MIR_X,x+2,y+8,dest);
  Intens_Out(kye_in,5,kye_index_i,col,INTENS_T | MIR_XY45 | MIR_Y | MIR_X,x+8,y+8,dest);
  Intens_Out(kye_shap,13,kye_index_s,kye_colors[2] | ((((dword)frame) & 0xFF00)<<16),INTENS_T | MIR_XY45,x+2,y+2,dest);
 }else{
  Kye_OB_Base(x, y, dest, 5, col, bcol);
  Intens_Out(kye_shap,13,kye_index_s,kye_colors[2] | ((((dword)frame) & 0xFF00)<<16),INTENS_T,x+2,y+2,dest);
 }
}


/*BlackRand
**Blacky randomizer: gives some pseudo - random spots in the blacky
**(an offset on a 16x16 img where 3 pixels borders are skipped)
**0x100 as ad resets the random sequence*/
adint kye_ob_blackrand_v=0;
adint Kye_OB_BlackRand(adint ad)
{
 adint x,y;
 if (ad==0x100) kye_ob_blackrand_v=0;
 kye_ob_blackrand_v+=ad|0xFF00;
 kye_ob_blackrand_v*=kye_ob_blackrand_v;
 x=(kye_ob_blackrand_v%10)+3;
 kye_ob_blackrand_v+=ad|0xFF00;
 kye_ob_blackrand_v*=kye_ob_blackrand_v;
 y=(kye_ob_blackrand_v%10)+3;
 return x+y*16;
}

/*Black hole
**rot specifies the eating animation frame when >64, otherwise the unoccupied
**frames*/
void Kye_O_Black(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 adint mix,t=Kye_OB_LinearSelect(rot&0xFF,256,32,&mix),i;
 dword colgry=Kye_OB_AddAlpha(kye_colors[5],(frame & 0xFF00)>>8),
       colblk=Kye_OB_AddAlpha(kye_colors[6],(frame & 0xFF00)>>8),
       colyel=Kye_OB_AddAlpha(kye_colors[2],(frame & 0xFF00)>>8),
       colred=Kye_OB_AddAlpha(kye_colors[3],(frame & 0xFF00)>>8),
       colblu=Kye_OB_AddAlpha(kye_colors[4],(frame & 0xFF00)>>8),
       tb,tm,tt;

 Kye_OB_MixBase(4, colblk, colgry);
 
 for (i=0;i<2;i++){
  Kye_OB_BlackRand(0x100);
  
  if (t>8){ /*Eating frames: change background*/
   switch(t){
    case 9:  tb=0xFFFFFF; break;
    case 10: tb=Kye_OB_MixColor(0xFFFFFF,colyel,144); break;
    case 11: tb=Kye_OB_MixColor(0xFFFFFF,colyel,80); break;
    case 12: tb=Kye_OB_MixColor(0xFFFFFF,colyel,32); break;
    case 13: tb=colyel; break;
    case 14: tb=Kye_OB_MixColor(colyel,colblu,192); break;
    case 15: tb=Kye_OB_MixColor(colyel,colblu,96); break;
    case 16: tb=Kye_OB_MixColor(colyel,colblu,32); break;
    case 17: tb=Kye_OB_MixColor(colyel,colblu,80); break;
    case 18: tb=Kye_OB_MixColor(colyel,colblu,160); break;
    case 19: tb=Kye_OB_MixColor(colyel,colblu,212); break;
    default:
     if (t<24) tb=Kye_OB_MixColor(colyel,colred,(24-t)<<6);
     else tb=Kye_OB_MixColor(colred,colblk,(31-t)<<5);
   }
   Intens_Out(kye_in,4,kye_index_i,tb,INTENS_T | MIR_XY45, 2,2,&kye_mixtmp[i]);
   Intens_Out(kye_in,4,kye_index_i,tb,INTENS_T | MIR_XY45 | MIR_Y,8,2,&kye_mixtmp[i]);
   Intens_Out(kye_in,4,kye_index_i,tb,INTENS_T | MIR_XY45 | MIR_X,2,8,&kye_mixtmp[i]);
   Intens_Out(kye_in,4,kye_index_i,tb,INTENS_T | MIR_XY45 | MIR_Y | MIR_X,8,8,&kye_mixtmp[i]);
  }
 
  if (t<9){ /*Make pixels dim*/
   tb=colblk;
   tm=128;
  }else if (t<13){ /*Fade in from white*/
   tb=0xFFFFFF;
   tm=256-((t-9)<<6);
  }else if (t<24){ /*Bright, original color*/
   tm=0;
  }else{ /*Fade out in dimness*/
   tb=colblk;
   tm=(t-24)<<4;
  }
  
  /*Note: t=0 must be identical to t=8 to have smooth animation*/
  tt=t;
  if (tt==0) tt=8;
  tt<<=3;
  kye_mixtmp[i].i[Kye_OB_BlackRand(tt)]=Kye_OB_MixColor(tb,colgry,tm);
  kye_mixtmp[i].i[Kye_OB_BlackRand(tt)]=Kye_OB_MixColor(tb,colgry,tm);
  kye_mixtmp[i].i[Kye_OB_BlackRand(tt)]=Kye_OB_MixColor(tb,colyel,tm);
  kye_mixtmp[i].i[Kye_OB_BlackRand(tt)]=Kye_OB_MixColor(tb,colyel,tm);
  kye_mixtmp[i].i[Kye_OB_BlackRand(tt)]=Kye_OB_MixColor(tb,colred,tm);
  kye_mixtmp[i].i[Kye_OB_BlackRand(tt)]=Kye_OB_MixColor(tb,colred,tm);
  kye_mixtmp[i].i[Kye_OB_BlackRand(tt)]=Kye_OB_MixColor(tb,colblu,tm);
  kye_mixtmp[i].i[Kye_OB_BlackRand(tt)]=Kye_OB_MixColor(tb,colblu,tm);

  /*A bit of 3D effect*/
  Intens_Out(kye_shap,12,kye_index_s,0xBB000000,INTENS_T,1,1,&kye_mixtmp[i]);
  Intens_Out(kye_shap,12,kye_index_s,0xBBFFFFFF,INTENS_T | MIR_Y | MIR_X,3,3,&kye_mixtmp[i]);
  
  t=(t+1)&0x1F;
 }
 
 Mix32(&kye_mixtmp[0],&kye_mixtmp[1],mix);
 Blit32(&kye_mixtmp[1],dest,0,0,x,y,16,16);

}


/*Virus
**Low 4 bits of frame specifies the variant: 
** 0: Blob
** 1: Snake
** 2: Eater (Gnasher)
** 3: Twister
** 4: Spike (Actually twister and spike does not differ too much)
**rot specifies the animation frame for all*/
void Kye_O_Virus(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 adint t, mix, i, xm, ym, t2, ino, flags;
 rot&=0xFF;

 Kye_OB_NewMix();

 switch (frame & 0xF){
 
  case 0: /*Blob*/
   t=Kye_OB_LinearSelect(rot,256,9,&mix); /*9 frames*/
   for (i=0;i<2;i++){
    Intens_Out(kye_big,t,kye_index_b,Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8),0,0,0,&kye_mixtmp[i]);
    Intens_Out(kye_shap,t+14,kye_index_s,Kye_OB_AddAlpha(kye_colors[7],(frame & 0xFF00)>>8),INTENS_T,2,2,&kye_mixtmp[i]);
    t=(t+1)%9;
   }
   break;

  case 1: /*Snake*/
   t=Kye_OB_LinearSelect(rot,256,6,&mix); /*6 frames*/
   for (i=0;i<2;i++){
    if (t>2) {xm=MIR_Y; ym=t-3+9;} else {xm=0; ym=t+9;}
    Intens_Out(kye_big,ym,kye_index_b,Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8),xm,0,0,&kye_mixtmp[i]);
    t=(t+1)%6;
   }
   break;
   
  case 2: /*Eater*/
   t=Kye_OB_LinearSelect(rot,256,9,&mix); /*6 + 3 frames*/
   for (i=0;i<2;i++){
    if (t>5) ym=((8-t)<<1)+6; else ym=t+6;
    Intens_Out(kye_out,ym,kye_index_o,Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8),0,0,0,&kye_mixtmp[i]);
    Intens_Out(kye_out,ym,kye_index_o,Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8),MIR_Y,8,0,&kye_mixtmp[i]);
    Intens_Out(kye_out,ym,kye_index_o,Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8),MIR_X | MIR_Y,8,8,&kye_mixtmp[i]);
    Intens_Out(kye_out,ym,kye_index_o,Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8),MIR_X,0,8,&kye_mixtmp[i]);
    t=(t+1)%9;
   }
   break;
   
  default: /*The two rotating virus (or any else dummy frame setting)*/
   t=Kye_OB_LinearSelect(rot,256,24,&mix); /*24 frames (traditional rotating stuff)*/
   for (i=0;i<2;i++){
    for (t2=0;t2<4;t2++){ /*4 quarters*/
     flags=0;
     switch (t%6){ /*Select image & flags*/
      case 0:  ino=12; break;
      case 1:  ino=13; break;
      case 2:  ino=14; break;
      case 3:  ino=15; break;
      case 4:  ino=14; flags|=MIR_XY45; break;
      default: ino=13; flags|=MIR_XY45;
     }
     switch (t2){
      case 0:  xm=0; ym=0; break;
      case 1:  xm=1; ym=0; if (flags & MIR_XY45) flags&=0xFFFFFFFF-MIR_XY45; else flags|=MIR_XY45; flags|=MIR_Y; break;
      case 2:  xm=1; ym=1; flags|=MIR_X | MIR_Y; break;
      default: xm=0; ym=1; if (flags & MIR_XY45) flags&=0xFFFFFFFF-MIR_XY45; else flags|=MIR_XY45; flags|=MIR_X;
     }
     Intens_Out(kye_out,ino,kye_index_o,Kye_OB_AddAlpha(kye_colors[0],(frame & 0xFF00)>>8),flags,xm<<3,ym<<3,&kye_mixtmp[i]);
     if ((frame & 0xF)==3) /*Draw inner portion*/
      Intens_Out(kye_in,ino,kye_index_i,Kye_OB_AddAlpha(kye_colors[3],(frame & 0xFF00)>>8),flags | INTENS_T,2+xm*6,2+ym*6,&kye_mixtmp[i]);
    }
    t=(t+1)%24;
   }
 }
 
 /*Send on - screen!*/

 Mix32(&kye_mixtmp[0],&kye_mixtmp[1],mix);
 Blit32(&kye_mixtmp[1],dest,0,0,x,y,16,16);

}


/*Door
**Low 5 bits of frame specifies the variant: 
** 4: Set if visible door
** 3: Uncovered flag for invisible door
** 2: Only Kye door
** 0,1: Direction: up (0), right (1), down (2), left (3)
**rot has no sense*/
void Kye_O_Door(sadint rot, sadint frame, sadint x, sadint y, tImage* dest)
{
 dword colk=Kye_OB_AddAlpha(kye_colors[12],(frame & 0xFF00)>>8),
       cola=Kye_OB_AddAlpha(kye_colors[13],(frame & 0xFF00)>>8);
 adint flags,i,t,xaff;

 if (((frame & 0x10)==0) && (frame & 0x8)){
  /*Uncover arrow*/
  switch (frame & 0x3){
   case 0:  flags=0; break;
   case 1:  flags=MIR_XY45 | MIR_Y; break;
   case 2:  flags=MIR_X; break;
   default: flags=MIR_XY45;
  }
  Intens_Out(kye_shap,7,kye_index_s,Kye_OB_AddAlpha(kye_colors[14],(frame & 0xFF00)>>8),flags | INTENS_T,x+2,y+2,dest);
 }
 
 /*Door graphic: depends on type*/
 switch (frame & 0x3){
  case 0: case 2: flags=MIR_XY45; t=MIR_Y; xaff=0; break;
  default: flags=0; t=MIR_X; xaff=1;
 }
 if (frame & 0x4){
  Intens_Out(kye_out,16,kye_index_o,colk,flags | INTENS_T,    x+(xaff<<2),y+((1-xaff)<<2),dest);
  Intens_Out(kye_out,16,kye_index_o,colk,flags | t | INTENS_T,x+(xaff<<2)+((1-xaff)<<3),y+((1-xaff)<<2)+(xaff<<3),dest);
 }else{
  for (i=0;i<2;i++){
   Intens_Out(kye_out,16,kye_index_o,cola,flags | INTENS_T,    x+(xaff<<3)*i,y+((1-xaff)<<3)*i,dest);
   Intens_Out(kye_out,16,kye_index_o,cola,flags | t | INTENS_T,x+(xaff<<3)*i+((1-xaff)<<3),y+((1-xaff)<<3)*i+(xaff<<3),dest);
  }
 }
 
 /*Similarly draw the directional arrow if the door is visible*/
 if (frame & 0x4) cola=colk;
 if (frame & 0x10){
  Intens_Out(kye_in,16,kye_index_i,cola,flags | INTENS_T,    x+(xaff<<2)+((1-xaff)<<1),y+((1-xaff)<<2)+(xaff<<1),dest);
  Intens_Out(kye_in,16,kye_index_i,cola,flags | t | INTENS_T,x+(xaff<<2)+((1-xaff)<<3),y+((1-xaff)<<2)+(xaff<<3),dest);
 }
}
