/*
** kyeobjs.c
**
** Functions to draw each object in kye
*/


#if !defined(KYEOBJS_H)
#define KYEOBJS_H

#include "baseinc.h"



/*Sets a color used in Kye.
**These are:
** 0: outer bound   (0x555555) (black)
** 1: kye color     (0x44EE22) (green)
** 2: base shape c. (0xEEDD11) (yellow)
** 3: arrows        (0xEE5522) (red)
** 4: auto slider   (0x2222EE) (blue)
** 5: walls         (0x888888) (gray)
** 6: black hole    (0x111111) (black)
** 7: blob          (0x7711EE) (purple)
** 8: viruses       (0x779977) (black)
** 9: diamond, 1    (0x2233EE) (blue)
**10: diamond, 2    (0x6666EE) (bright blue)
**11: background    (0x000000) (white)
**12: Kye door      (0x66EE3311) (red)
**13: All ob. door  (0x775555EE)
**14: Uncover arrow (0x8833EE33)
*/
void Kye_Color(byte i, dword color);

/*Sets up index tables & mixer
**(Without index tables the game can work, but the mixer is necessary)
**returns nonzero if mixer could not be allocated*/
adint Kye_OB_Alloc();

/*Frees allocated stuff. Must be called before exitting to prevent leaks*/
void Kye_OB_Free();


/*General for objects:
**rot: 0-256, rotation of the object. 0 points upwards, rotating is to the
**right. 256 equals to 360 degrees.
**x and y and dest should be obvious. x and y points at the upper left corner
**of the 16x16 box which contains the object.
**frame is for playing special object - related animation, 0-256 the same way
*/

/*Kye
**neither rot and frame does anything*/
void Kye_O_Kye(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Slider
**frame does not do anything*/
void Kye_O_Slider(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Rocky
**frame does not do anything*/
void Kye_O_Rocky(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Auto - Slider
**frame does not do anything*/
void Kye_O_ASlider(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Auto - Rocky
**frame does not do anything*/
void Kye_O_ARocky(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Clocker
**frame specifies if anticlocker: 0 for clocker, else for anticlocker
**rot does nothing*/
void Kye_O_Clocker(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Block
**Neither frame or rot does anything*/
void Kye_O_Block(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*RBlock
**Neither frame or rot does anything*/
void Kye_O_RBlock(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*EBlock (Eatable block)
**Neither frame or rot does anything*/
void Kye_O_EBlock(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*ERBlock
**Neither frame or rot does anything*/
void Kye_O_ERBlock(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Wall
**frame is a bitmask of the rounded corners in order:
**up left, up right, bottom right, bottom left*/
void Kye_O_Wall(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Mover
**frame: top 4 bits: reverse direction animation*/
void Kye_O_Mover(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Diamond
**rot chooses which sides should "shine" (0-255)*/
void Kye_O_Diamond(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*TBlock (Timer block)
**frame does not do anything, rot is for the countdown (255: full, 0: counted down)*/
void Kye_O_TBlock(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Magnet
**rot specifies the direction: 0: up - down; 64: left - right etc.*/
void Kye_O_Magnet(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Black hole
**rot specifies the eating animation frame when >64, otherwise the unoccupied
**frames*/
void Kye_O_Black(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Virus
**Low 4 bits of frame specifies the variant: 
** 0: Blob
** 1: Snake
** 2: Eater (Gnasher)
** 3: Twister
** 4: Spike (Actually twister and spike does not differ too much)
**rot specifies the animation frame for all*/
void Kye_O_Virus(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);

/*Door
**Low 5 bits of frame specifies the variant: 
** 4: Set if visible door
** 3: Uncovered flag for invisible door
** 2: Only Kye door
** 0,1: Direction: up (0), right (1), down (2), left (3)
**rot has no sense*/
void Kye_O_Door(sadint rot, sadint frame, sadint x, sadint y, tImage* dest);


#endif




