/*
** levelprc.c
**
** Kye level processor: sets up the objects and map elements according
** to the information read out either from standard Kye levels, or
** RuChKye levels
*/


#if !defined(LEVELPRC_H)
#define LEVELPRC_H


#include "baseinc.h"



/*Level_KyeBlock
**Translates a standard Kye level block, and returns it as a RuChKye block
**x and y are for x,y coordinates: some objects (auto - rocky, slider) rely
**on it in the original*/
void Level_KyeBlock(byte inchar, adint x, adint y, byte* tchar, byte* rchar, byte* achar);

/*Level_SkipLine
**Skips a line in the input (reads until passing over \n)*/
void Level_SkipLine(byte (*feeder) ());

/*Level_GetNumLine
**Parses a line to it's end which supposed to be numeric, and returns the
**number could be extracted, 0 if nothing numeric, 0xFFFF if the "RuChKye Level"
**string was experienced. Feeder is the function which feeds it with
**characters, 0xFF on eof*/
adint Level_GetNumLine(byte (*feeder) ());

/*Level_RuChKyeBlock
**Translates a RuChKye level block
**tchar: Type selector
**rchar: Rotation
**  U/u/T/t/N/n/8/^/A: Up (Top, North)
**  L/l/    E/e/6/>:   Left (East)
**  D/d/B/b/S/s/2/v/V: Down (Bottom, South)
**  R/r/    W/w/4/<:   Right (West)
**achar: Attributes
**xi and yi are the coordinates in the input, mx and my are where the block
**supposed to go. These are seperated to properly interpret original kye
**style type specifications (regardless to where the map will go on the
**playing field)
**returns 0 unless Kye was placed (1), or a diamond (2)
*/
adint Level_RuChKyeBlock(byte tchar, byte rchar, byte achar, adint xi, adint yi, adint mx, adint my);

/*Outputs an original Kye level on the middle of the playing field.
**The feeder should start feeding at the first line of the level data
**returns 0 if the level was all OK, 1 if it is buggy, but might be playable,
**2 if it is a complete mess (no or more Kyes, or no diamonds)*/
/*(Original Kye had 30x20 maps)*/
adint Level_Kye(byte (*feeder) ());

/*Similar to the above for RuChKye level data
**sx and sy are the map sizes. However in RuChKye maps it is allowed for a line
**to be shorter than the map's size*/
adint Level_RuChKye(byte (*feeder) (), adint sx, adint sy);


#endif
