/*
** levelrdc.c
**
** Kye level resource decoder: actually a simple PCX decoder for 7bit ASCII
** text where 8th bit is always unused
**
** Format:
**  0-127: ASCII letter
**  128-255: Repeat following:
**   7th bit is 1
**   6-5 bit: sequence length: 1-4 (characters)
**   4-0 bit: repeat count: 1-32 (repeats)
*/


#include "baseinc.h"
#include "levelrdc.h"



/*ASCII7_Start
**Getting a buffer and it's length this sets up the decoder to decode.
**(Starting decoding without buffer will return 0xFF)*/
byte* ascii7_buf=NULL;
adint ascii7_len=0;
void ASCII7_Start(byte* buf, adint len)
{
 ascii7_buf=buf;
 ascii7_len=len;
}


/*ASCII7_Getch
**Gets one character (0-127) from the buffer, or 0xFF (255) on EOF
*/
adint ascii7_repc=0;    /*How many times to repeat the current sequence*/
adint ascii7_repp=0xFF; /*Pointer in the repeated character sequence (0-3)*/
byte ASCII7_Getch()
{
 byte ret;
 adint tlen;

 if (ascii7_len==0) return 0xFF;
 
 if ((*ascii7_buf)&0x80){ /*Longer sequences*/

  tlen=(((*ascii7_buf)>>5)& 0x3);
 
  if (ascii7_repp==0xFF){ /*Read in new sequence*/
   if (ascii7_len<(tlen+2)){ /*Erratic*/
    ascii7_len=0;
    return 0xFF;
   }
   ascii7_repc=((*ascii7_buf)&0x1F)+1; /*Set up counters*/
   ascii7_repp=0;
  }

  ascii7_repp++;  
  ret=ascii7_buf[ascii7_repp];
  
  if (ascii7_repp>tlen){ /*Sequence ended*/
   ascii7_repc--;
   ascii7_repp=0;
   if (ascii7_repc==0){ /*End of the repeat block*/
    ascii7_len-=tlen+2;
    ascii7_buf+=tlen+2;
    ascii7_repp=0xFF;
   }
  }
  
  return ret; /*The character got*/

 }else{ /*Return single char*/
  ascii7_len--;
  ascii7_buf++;
  return *(ascii7_buf-1);
 }
}
