/*
** screen.c
**
** SDL screen handling module
**
** Set up a 32bit mode, lock, unlock, and update routines.
**
*/


#include "baseinc.h"
#include "screen.h"  /*Function prototypes*/

#include <SDL/SDL.h> /*SDL*/




SDL_Surface*  screen_memory; /*The memory on which drawing is done*/
SDL_Surface*  screen_video;  /*The actual display surface*/
tImage        screen_memory_timage;



sadint Screen_Set(adint x, adint y)
{
 adint i=32;

 /*Try to set up the mode in the highest possible bit depth
 **(No problem if not 32 as SDL can do the job when copiing the software
 **surface*/
 while(i>0){
  screen_video=SDL_SetVideoMode(x,y,i,/*SDL_FULLSCREEN |*/ SDL_HWSURFACE);
  if (screen_video!=NULL) break; /*OK! Proper mode found, halt here!*/
  i-=8;
 }
 if (i==0) return -1;
 /*If i could reach 0, it means nothing could be set up... Trouble!*/
 
 /*Allocate the software surface in traditional 0RGB format*/
 screen_memory=SDL_CreateRGBSurface(SDL_SWSURFACE,x,y,32,0xFF0000,0xFF00,0xFF,0);
 if (screen_memory==NULL){ /*Problem!*/
  SDL_FreeSurface(screen_video);
  return -1;
 }
 
 SDL_ShowCursor(SDL_DISABLE); /*No need of mouse cursor*/
 screen_memory_timage.x=x;
 screen_memory_timage.y=y;
 screen_memory_timage.c=NULL;
 
 return 0; /*Everything went fine!*/
}


void Screen_Free()
{
 SDL_FreeSurface(screen_video);
 SDL_FreeSurface(screen_memory);
}


tImage* Screen_Lock()
{
 if (SDL_MUSTLOCK(screen_memory)) if (SDL_LockSurface(screen_memory)<0) return NULL;
 /*Do the lock if it is necessary on the surface*/
 screen_memory_timage.p=((adint)(screen_memory->pitch))>>2;
 screen_memory_timage.i=(dword*)(screen_memory->pixels);
 return &screen_memory_timage;
}


void Screen_Unlock()
{
 if (SDL_MUSTLOCK(screen_memory)) SDL_UnlockSurface(screen_memory);
}


void Screen_Update(sadint x, sadint y, sadint w, sadint h)
{
 SDL_Rect urect;
 /*Do value - checks, and fixes. SDL seems to not check them*/
 if (x<0){ w+=x; x=0; }
 if (y<0){ h+=y; y=0; }
 if (x+w>screen_memory_timage.x) w=screen_memory_timage.x-x;
 if (y+h>screen_memory_timage.y) h=screen_memory_timage.y-y;
 if ((w<=0) || (h<=0)) return;
 /*They are OK, it can go*/
 urect.x=x;
 urect.y=y;
 urect.w=w;
 urect.h=h;
 SDL_BlitSurface(screen_memory,&urect,screen_video,&urect);
 SDL_UpdateRect(screen_video,x,y,w,h);
}
