/*
** text.c
**
** For writing text on screen
*/


#include "baseinc.h"
#include "intens.h"
#include "text_fnt.h"
#include "text.h"


#define TEXT_HEIGHT 16
#define TEXT_SPACING 2


byte *text_extended=NULL;
/*Extended font set (ascii 128-255)*/
adint *text_basei=NULL;
adint *text_exti=NULL;
/*Indexes for the base set & the extended set*/


/*Returns image number which the letter should become. If an extended set
**is present, returns 128-255 for those characters, otherwise 95 (127, but
**the standard set has the first 32 images cropped). -1 stands for
**characters which should be ignored in display*/
sadint Text_Image(byte c)
{
 if (c<32) return -1;
 if (c<128) return c-32;
 if (text_extended==NULL) return 95;
 return c;
}


/*Allocates memory if necessary and sets the extended charset if non - null
**is passed. The module works without this, but sloow.*/
void Text_Alloc(byte* ext)
{
 if (ext!=NULL) text_extended=ext;
 if (text_basei==NULL){
  if ((text_basei=malloc(sizeof(adint)*512))!=NULL){
   text_exti=text_basei+256;
   Intens_Index(text_fnt,text_basei);
  }
 }
 if ((text_extended!=NULL) && (text_exti!=NULL)) Intens_Index(text_extended,text_exti);
}

/*Frees memory allocated to the text module. Call when exitting*/
void Text_Free()
{
 free(text_basei);
 text_basei=NULL;
 text_exti=NULL;
}

/*Prints a null terminated string beginning at the given location*/
adint Text_Print(byte* str, sadint x, sadint y, dword col, tImage*dest)
{
 sadint t=0,xc=x;
 sadint im;
 while (str[t]!=0){
  if (str[t]=='\n'){
   xc=x; y+=TEXT_HEIGHT; t++;
   continue;
  }
  im=Text_Image(str[t]);
  if ((im>=0) && (im<96)) xc+=((Intens_Out(text_fnt,im,text_basei,col,INTENS_T,xc,y,dest))&0xFF)+TEXT_SPACING;
  else if (im>=128) xc+=((Intens_Out(text_extended,im-128,text_exti,col,INTENS_T,xc,y,dest))&0xFF)+TEXT_SPACING;
  t++;
 }
 return xc-TEXT_SPACING-x;
}


/*Used by the other functions to output a chain of characters*/
adint Text_Out(byte* str, adint len, sadint x, sadint y, dword col, tImage* dest)
{
 sadint im,t=0,xc=0;
 while (t<len){
  im=Text_Image(str[t]);
  if (im<128){ if (im!=-1) xc+=((Intens_Out(text_fnt,im,text_basei,col,INTENS_T,xc+x,y,dest))&0xFF)+TEXT_SPACING; }
  else xc+=((Intens_Out(text_extended,im,text_exti,col,INTENS_T,xc+x,y,dest))&0xFF)+TEXT_SPACING;
  t++;
 }
 return xc-TEXT_SPACING;
}


/*Outputs aligned text. Returns the height of the block in pixels. If NULL
**is passed as dest, then only calculates height. Uses the full width of the
**passed tImage (create "window"s to limit). Align can be 'c', 'l', 'r', or
**'j'*/
adint Text_Align(byte* str, byte align, sadint y, dword col, tImage*dest)
{
 adint t=0, ts=0, te=0, tj, wr, xc=0, xco=0, yc=y, wrl, nl=0, wc=0;
 sadint im;
 
 while (str[t]!=0){
 
  /*Get next word's width*/
  wr=0;
  wrl=0;
  while (str[t+wr]>32){ /*Some letter comes*/
   im=Text_Image(str[t+wr]);
   if (im<128){ if (im!=-1) wrl+=((Intens_Out(text_fnt,im,text_basei,0,INTENS_T,0,0,NULL))&0xFF)+TEXT_SPACING; }
   else wrl+=((Intens_Out(text_extended,im,text_exti,0,INTENS_T,0,0,NULL))&0xFF)+TEXT_SPACING;
   wr++;
  }
  wrl-=TEXT_SPACING; /*Proper word width got!*/
  if (xc==0){ xc=wrl; xco=xc; t+=wr; te=t; wc++; } /*At least 1 word per line*/
  else{
   wrl+=TEXT_SPACING;
   /*Fits in?*/
   if ((xc+wrl)<=(dest->x)){ xc+=wrl; xco=xc; t+=wr; te=t; wc++; }
   else nl=1;
  }
  
  /*Process a sequence of control chars & spaces*/
  if (nl==0){
   wr=0;
   wrl=0;
   while (str[t+wr]<=32){
    if (str[t+wr]==' '){ wrl+=((Intens_Out(text_fnt,0,text_basei,0,INTENS_T,0,0,NULL))&0xFF)+TEXT_SPACING; }
    else if (str[t+wr]=='\n') { nl=1; wr++; t+=wr; if (align=='j') align='J'; break; }
    else if (str[t+wr]==0) { nl=1; if (align=='j') align='J'; break; }
    wr++;
   }
   if (nl==0){ if ((xc+wrl)<=(dest->x)){ xc+=wrl; t+=wr; } else nl=1; }
  }
  
  /*Write out line when necessary*/
  if (nl){

   /*Create output*/
   if (dest!=NULL){
    /*xco holds the length in pixels... Start position?*/
    if (align!='j'){
     switch (align){
      case 'c': xc=((dest->x)-xco)>>1; break;
      case 'r': xc=((dest->x)-xco); break;
      case 'J': align='j'; /* + align left*/
      default:  xc=0;
     }
     Text_Out(str+ts,te-ts,xc,yc,col,dest);
    }else{
     /*Output it word by word*/
     /*For this, restart from ts, rescan, and output*/
     tj=ts;
     xc=0;
     xco-=(wc-1)*TEXT_SPACING; /*Length without the spacings between words*/
     wrl=(dest->x)-xco; /*Total pixels for spaces*/
     while (tj<te){
      wr=0;
      while ((str[tj+wr]<=32) && ((tj+wr)<te)) wr++; /*One word + front spaces*/
      while ((str[tj+wr]>32)  && ((tj+wr)<te)) wr++;
      xc+=Text_Out(str+tj,wr,xc,yc,col,dest);
      wc--; if (wc>0){ im=wrl/wc; xc+=im; wrl-=im; }
      tj+=wr;
     }
    }
   }
   
   /*Get in next line*/
   nl=0; xco=0; xc=0; ts=t; te=t; wc=0;
   yc+=TEXT_HEIGHT;
  }
  
 }
 
 return (yc-y)*TEXT_HEIGHT;
}
