/*
** timer.c
**
** Event based timing
*/


#include "baseinc.h"
#include "timer.h"

#include <SDL/SDL.h>


#define TIMER_USE_NEW


adint timer_ticks=0;
adint timer_msec=0;

adint Timer_Time(sadint change)
{
 if ((change<0) && (timer_ticks<change)) timer_ticks=0;
 else timer_ticks+=change;
 return timer_ticks;
}

void Timer_Reset()
{
 timer_ticks=0;
}

adint Timer_Get()
{
 return timer_msec;
}



#if !defined(TIMER_USE_NEW)


Uint32 Timer_EventPusher(Uint32 interval)
{
 SDL_Event timeevent;
 timeevent.type=SDL_USEREVENT;
 timeevent.user.code=TIMER_CODE;
 SDL_PushEvent(&timeevent); /*Send this event!*/ 
 timer_ticks++;
 return interval; /*Continue ticking!*/
}

sadint Timer_Set(adint msec)
{
 if (timer_msec) SDL_SetTimer(0,NULL);
 timer_msec=0;
 if (msec==0) return 0; /*Clear timer*/
 /*Set timer*/
 /*(Strange that the doc. does not talk about what if it fails...)*/
 SDL_SetTimer(msec,&Timer_EventPusher);
 timer_msec=msec;
 return 0;
}


#else


SDL_TimerID timer_id=NULL;

Uint32 Timer_EventPusher(Uint32 interval, void* p)
{
 SDL_Event timeevent;
 timeevent.type=SDL_USEREVENT;
 timeevent.user.code=TIMER_CODE;
 SDL_PushEvent(&timeevent); /*Send this event!*/ 
 timer_ticks++;
 return interval; /*Continue ticking!*/
}

sadint Timer_Set(adint msec)
{
 if (timer_id!=NULL) SDL_RemoveTimer(timer_id);
 timer_msec=0;
 if (msec==0) return 0; /*No new timer needed*/
 /*Set timer*/
 if ((timer_id=SDL_AddTimer(msec,&Timer_EventPusher,NULL))==NULL) return -1;
 timer_msec=msec;
 return 0;
}


#endif
